encapsulated package CodegenUtil
"
  file:        CodegenUtil.mo
  package:     CodegenUtil
  description: Generated by Susan.
"

public import Tpl;

public import ExpressionDumpTpl;
public import SimCodeVar;
public import SimCode;
public import BackendDAE;
public import System;
public import Absyn;
public import DAE;
public import ClassInf;
public import Util;
public import Expression;
public import ExpressionDump;
public import Error;
public import Types;

public function symbolName
  input Tpl.Text txt;
  input String a_modelNamePrefix;
  input String a_symbolName;

  output Tpl.Text out_txt;
algorithm
  out_txt := Tpl.writeStr(txt, a_modelNamePrefix);
  out_txt := Tpl.writeTok(out_txt, Tpl.ST_STRING("_"));
  out_txt := Tpl.writeStr(out_txt, a_symbolName);
end symbolName;

public function replaceDotAndUnderscore
  input Tpl.Text in_txt;
  input String in_a_str;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_str)
    local
      Tpl.Text txt;
      String i_name;
      String ret_4;
      String ret_3;
      Tpl.Text l_str__underscores;
      String ret_1;
      Tpl.Text l_str__dots;

    case ( txt,
           i_name )
      equation
        ret_1 = System.stringReplace(i_name, ".", "_");
        l_str__dots = Tpl.writeStr(Tpl.emptyTxt, ret_1);
        ret_3 = System.stringReplace(Tpl.textString(l_str__dots), "_", "__");
        l_str__underscores = Tpl.writeStr(Tpl.emptyTxt, ret_3);
        ret_4 = System.unquoteIdentifier(Tpl.textString(l_str__underscores));
        txt = Tpl.writeStr(txt, ret_4);
      then txt;
  end match;
end replaceDotAndUnderscore;

public function getGeneralTarget
  input Tpl.Text in_txt;
  input String in_a_str;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_str)
    local
      Tpl.Text txt;
      String i_str;

    case ( txt,
           "msvc10" )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("msvc"));
      then txt;

    case ( txt,
           "msvc12" )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("msvc"));
      then txt;

    case ( txt,
           "msvc13" )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("msvc"));
      then txt;

    case ( txt,
           "msvc15" )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("msvc"));
      then txt;

    case ( txt,
           "msvc19" )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("msvc"));
      then txt;

    case ( txt,
           i_str )
      equation
        txt = Tpl.writeStr(txt, i_str);
      then txt;
  end match;
end getGeneralTarget;

public function underscorePath
  input Tpl.Text in_txt;
  input Absyn.Path in_a_path;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_path)
    local
      Tpl.Text txt;
      String i_name_1;
      Absyn.Path i_path;
      Absyn.Ident i_name;

    case ( txt,
           Absyn.QUALIFIED(name = i_name, path = i_path) )
      equation
        txt = replaceDotAndUnderscore(txt, i_name);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("_"));
        txt = underscorePath(txt, i_path);
      then txt;

    case ( txt,
           Absyn.IDENT(name = i_name_1) )
      equation
        txt = replaceDotAndUnderscore(txt, i_name_1);
      then txt;

    case ( txt,
           Absyn.FULLYQUALIFIED(path = i_path) )
      equation
        txt = underscorePath(txt, i_path);
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end underscorePath;

public function crefStr
  input Tpl.Text in_txt;
  input DAE.ComponentRef in_a_cr;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_cr)
    local
      Tpl.Text txt;
      DAE.ComponentRef i_componentRef;
      list<DAE.Subscript> i_subscriptLst;
      DAE.Ident i_ident;
      String ret_1;
      String ret_0;

    case ( txt,
           DAE.CREF_IDENT(ident = i_ident, subscriptLst = i_subscriptLst) )
      equation
        ret_0 = System.unquoteIdentifier(i_ident);
        txt = Tpl.writeStr(txt, ret_0);
        txt = subscriptsStr(txt, i_subscriptLst);
      then txt;

    case ( txt,
           DAE.CREF_QUAL(ident = "$DER", componentRef = i_componentRef) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("der("));
        txt = crefStr(txt, i_componentRef);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(")"));
      then txt;

    case ( txt,
           DAE.CREF_QUAL(ident = "$CLKPRE", componentRef = i_componentRef) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("previous("));
        txt = crefStr(txt, i_componentRef);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(")"));
      then txt;

    case ( txt,
           DAE.CREF_QUAL(ident = i_ident, subscriptLst = i_subscriptLst, componentRef = i_componentRef) )
      equation
        ret_1 = System.unquoteIdentifier(i_ident);
        txt = Tpl.writeStr(txt, ret_1);
        txt = subscriptsStr(txt, i_subscriptLst);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("._"));
        txt = crefStr(txt, i_componentRef);
      then txt;

    case ( txt,
           _ )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("CREF_NOT_IDENT_OR_QUAL"));
      then txt;
  end match;
end crefStr;

public function crefStrNoUnderscore
  input Tpl.Text in_txt;
  input DAE.ComponentRef in_a_cr;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_cr)
    local
      Tpl.Text txt;
      DAE.ComponentRef i_componentRef;
      list<DAE.Subscript> i_subscriptLst;
      DAE.Ident i_ident;

    case ( txt,
           DAE.CREF_IDENT(ident = i_ident, subscriptLst = i_subscriptLst) )
      equation
        txt = Tpl.writeStr(txt, i_ident);
        txt = subscriptsStr(txt, i_subscriptLst);
      then txt;

    case ( txt,
           DAE.CREF_QUAL(ident = "$DER", componentRef = i_componentRef) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("der("));
        txt = crefStrNoUnderscore(txt, i_componentRef);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(")"));
      then txt;

    case ( txt,
           DAE.CREF_QUAL(ident = "$CLKPRE", componentRef = i_componentRef) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("previous("));
        txt = crefStrNoUnderscore(txt, i_componentRef);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(")"));
      then txt;

    case ( txt,
           DAE.CREF_QUAL(ident = i_ident, subscriptLst = i_subscriptLst, componentRef = i_componentRef) )
      equation
        txt = Tpl.writeStr(txt, i_ident);
        txt = subscriptsStr(txt, i_subscriptLst);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("."));
        txt = crefStrNoUnderscore(txt, i_componentRef);
      then txt;

    case ( txt,
           _ )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("CREF_NOT_IDENT_OR_QUAL"));
      then txt;
  end match;
end crefStrNoUnderscore;

protected function lm_45
  input Tpl.Text in_txt;
  input list<DAE.Subscript> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<DAE.Subscript> rest;
      DAE.Subscript i_s;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_s :: rest )
      equation
        txt = subscriptStr(txt, i_s);
        txt = Tpl.nextIter(txt);
        txt = lm_45(txt, rest);
      then txt;
  end match;
end lm_45;

public function subscriptsStr
  input Tpl.Text in_txt;
  input list<DAE.Subscript> in_a_subscripts;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_subscripts)
    local
      Tpl.Text txt;
      list<DAE.Subscript> i_subscripts;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_subscripts )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("["));
        txt = Tpl.pushIter(txt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_STRING(",")), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        txt = lm_45(txt, i_subscripts);
        txt = Tpl.popIter(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("]"));
      then txt;
  end match;
end subscriptsStr;

public function crefStrMatlabSafe
  input Tpl.Text in_txt;
  input DAE.ComponentRef in_a_cr;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_cr)
    local
      Tpl.Text txt;
      DAE.ComponentRef i_componentRef;
      list<DAE.Subscript> i_subscriptLst;
      DAE.Ident i_ident;

    case ( txt,
           DAE.CREF_IDENT(ident = i_ident, subscriptLst = i_subscriptLst) )
      equation
        txt = Tpl.writeStr(txt, i_ident);
        txt = subscriptsStrMatlabSafe(txt, i_subscriptLst);
      then txt;

    case ( txt,
           DAE.CREF_QUAL(ident = "$DER", componentRef = i_componentRef) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("der_"));
        txt = crefStrMatlabSafe(txt, i_componentRef);
      then txt;

    case ( txt,
           DAE.CREF_QUAL(ident = "$CLKPRE", componentRef = i_componentRef) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("pre_"));
        txt = crefStrMatlabSafe(txt, i_componentRef);
      then txt;

    case ( txt,
           DAE.CREF_QUAL(ident = i_ident, subscriptLst = i_subscriptLst, componentRef = i_componentRef) )
      equation
        txt = Tpl.writeStr(txt, i_ident);
        txt = subscriptsStrMatlabSafe(txt, i_subscriptLst);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("_"));
        txt = crefStrMatlabSafe(txt, i_componentRef);
      then txt;

    case ( txt,
           _ )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("CREF_NOT_IDENT_OR_QUAL"));
      then txt;
  end match;
end crefStrMatlabSafe;

protected function lm_48
  input Tpl.Text in_txt;
  input list<DAE.Subscript> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<DAE.Subscript> rest;
      DAE.Subscript i_s;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_s :: rest )
      equation
        txt = subscriptStr(txt, i_s);
        txt = Tpl.nextIter(txt);
        txt = lm_48(txt, rest);
      then txt;
  end match;
end lm_48;

public function subscriptsStrMatlabSafe
  input Tpl.Text in_txt;
  input list<DAE.Subscript> in_a_subscripts;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_subscripts)
    local
      Tpl.Text txt;
      list<DAE.Subscript> i_subscripts;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_subscripts )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
        txt = Tpl.pushIter(txt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_STRING(",")), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        txt = lm_48(txt, i_subscripts);
        txt = Tpl.popIter(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(")"));
      then txt;
  end match;
end subscriptsStrMatlabSafe;

public function subscriptStr
  input Tpl.Text in_txt;
  input DAE.Subscript in_a_subscript;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_subscript)
    local
      Tpl.Text txt;
      DAE.Exp i_exp;
      Absyn.Path i_n;
      Integer i_i;
      String ret_2;
      String ret_1;
      String ret_0;

    case ( txt,
           DAE.INDEX(exp = DAE.ICONST(integer = i_i)) )
      equation
        txt = Tpl.writeStr(txt, intString(i_i));
      then txt;

    case ( txt,
           DAE.INDEX(exp = DAE.ENUM_LITERAL(name = i_n)) )
      equation
        txt = dotPath(txt, i_n);
      then txt;

    case ( txt,
           DAE.INDEX(exp = (i_exp as DAE.CREF(componentRef = _))) )
      equation
        ret_0 = ExpressionDump.printExpStr(i_exp);
        txt = Tpl.writeStr(txt, ret_0);
      then txt;

    case ( txt,
           DAE.SLICE(exp = DAE.ICONST(integer = i_i)) )
      equation
        txt = Tpl.writeStr(txt, intString(i_i));
      then txt;

    case ( txt,
           DAE.INDEX(exp = i_exp) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("UNKNOWN_SUBSCRIPT /* "));
        ret_1 = ExpressionDump.printExpStr(i_exp);
        txt = escapeCComments(txt, ret_1);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(" */"));
      then txt;

    case ( txt,
           DAE.SLICE(exp = i_exp) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("UNKNOWN_SUBSCRIPT /* "));
        ret_2 = ExpressionDump.printExpStr(i_exp);
        txt = escapeCComments(txt, ret_2);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(" */"));
      then txt;

    case ( txt,
           DAE.WHOLEDIM() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("WHOLEDIM"));
      then txt;

    case ( txt,
           DAE.WHOLE_NONEXP() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("WHOLE_NONEXP"));
      then txt;

    case ( txt,
           _ )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("UNKNOWN_SUBSCRIPT"));
      then txt;
  end match;
end subscriptStr;

public function escapeCComments
  input Tpl.Text txt;
  input String a_stringWithCComments;

  output Tpl.Text out_txt;
protected
  String ret_1;
  String ret_0;
algorithm
  ret_0 := System.stringReplace(a_stringWithCComments, "/*", "(*");
  ret_1 := System.stringReplace(ret_0, "*/", "*)");
  out_txt := Tpl.writeStr(txt, ret_1);
end escapeCComments;

public function initDefaultValXml
  input Tpl.Text in_txt;
  input DAE.Type in_a_type__;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_type__)
    local
      Tpl.Text txt;
      DAE.Type i_type__;
      Tpl.Text txt_0;
      String ret_0;

    case ( txt,
           DAE.T_INTEGER(varLst = _) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("0"));
      then txt;

    case ( txt,
           DAE.T_REAL(varLst = _) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("0.0"));
      then txt;

    case ( txt,
           DAE.T_BOOL(varLst = _) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("false"));
      then txt;

    case ( txt,
           DAE.T_STRING(varLst = _) )
      then txt;

    case ( txt,
           DAE.T_ENUMERATION(index = _) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("0"));
      then txt;

    case ( txt,
           i_type__ )
      equation
        txt_0 = Tpl.writeTok(Tpl.emptyTxt, Tpl.ST_STRING("initial value of unknown type: "));
        ret_0 = Types.unparseType(i_type__);
        txt_0 = Tpl.writeStr(txt_0, ret_0);
        txt = error(txt, Tpl.sourceInfo("CodegenUtil.tpl", 186, 14), Tpl.textString(txt_0));
      then txt;
  end match;
end initDefaultValXml;

protected function fun_53
  input Tpl.Text in_txt;
  input Boolean in_a_bool;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_bool)
    local
      Tpl.Text txt;

    case ( txt,
           false )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("false"));
      then txt;

    case ( txt,
           _ )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("true"));
      then txt;
  end match;
end fun_53;

public function initValXml
  input Tpl.Text in_txt;
  input DAE.Exp in_a_exp;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_exp)
    local
      Tpl.Text txt;
      DAE.Exp i_exp;
      Integer i_index;
      Boolean i_bool;
      String i_string;
      Real i_real;
      Integer i_integer;
      Tpl.Text txt_1;
      String ret_0;

    case ( txt,
           DAE.ICONST(integer = i_integer) )
      equation
        txt = Tpl.writeStr(txt, intString(i_integer));
      then txt;

    case ( txt,
           DAE.RCONST(real = i_real) )
      equation
        txt = Tpl.writeStr(txt, realString(i_real));
      then txt;

    case ( txt,
           DAE.SCONST(string = i_string) )
      equation
        ret_0 = Util.escapeModelicaStringToXmlString(i_string);
        txt = Tpl.writeStr(txt, ret_0);
      then txt;

    case ( txt,
           DAE.BCONST(bool = i_bool) )
      equation
        txt = fun_53(txt, i_bool);
      then txt;

    case ( txt,
           DAE.ENUM_LITERAL(index = i_index) )
      equation
        txt = Tpl.writeStr(txt, intString(i_index));
      then txt;

    case ( txt,
           i_exp )
      equation
        txt_1 = Tpl.writeTok(Tpl.emptyTxt, Tpl.ST_STRING("initial value of unknown type: "));
        txt_1 = ExpressionDumpTpl.dumpExp(txt_1, i_exp, "\"");
        txt = error(txt, Tpl.sourceInfo("CodegenUtil.tpl", 197, 14), Tpl.textString(txt_1));
      then txt;
  end match;
end initValXml;

public function getVariablity
  input Tpl.Text in_txt;
  input BackendDAE.VarKind in_a_varKind;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_varKind)
    local
      Tpl.Text txt;

    case ( txt,
           BackendDAE.DISCRETE() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("discrete"));
      then txt;

    case ( txt,
           BackendDAE.PARAM() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("parameter"));
      then txt;

    case ( txt,
           BackendDAE.CONST() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("constant"));
      then txt;

    case ( txt,
           _ )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("continuous"));
      then txt;
  end match;
end getVariablity;

public function variabilityString
  input Tpl.Text in_txt;
  input BackendDAE.VarKind in_a_varKind;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_varKind)
    local
      Tpl.Text txt;
      Absyn.Path i_fullClassName;
      DAE.ComponentRef i_dcr;
      Integer i_index;

    case ( txt,
           BackendDAE.VARIABLE() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("variable"));
      then txt;

    case ( txt,
           BackendDAE.STATE(derName = NONE(), index = i_index) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("STATE("));
        txt = Tpl.writeStr(txt, intString(i_index));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(")"));
      then txt;

    case ( txt,
           BackendDAE.STATE(derName = SOME(i_dcr), index = i_index) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("STATE("));
        txt = Tpl.writeStr(txt, intString(i_index));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(","));
        txt = crefStrNoUnderscore(txt, i_dcr);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(")"));
      then txt;

    case ( txt,
           BackendDAE.STATE_DER() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("STATE_DER"));
      then txt;

    case ( txt,
           BackendDAE.DUMMY_DER() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("DUMMY_DER"));
      then txt;

    case ( txt,
           BackendDAE.DUMMY_STATE() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("DUMMY_STATE"));
      then txt;

    case ( txt,
           BackendDAE.CLOCKED_STATE(previousName = _) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("CLOCKED_STATE"));
      then txt;

    case ( txt,
           BackendDAE.DISCRETE() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("DISCRETE"));
      then txt;

    case ( txt,
           BackendDAE.PARAM() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("PARAM"));
      then txt;

    case ( txt,
           BackendDAE.CONST() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("CONST"));
      then txt;

    case ( txt,
           BackendDAE.EXTOBJ(fullClassName = i_fullClassName) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("EXTOBJ: "));
        txt = dotPath(txt, i_fullClassName);
      then txt;

    case ( txt,
           BackendDAE.JAC_VAR() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("JACOBIAN_VAR"));
      then txt;

    case ( txt,
           BackendDAE.JAC_DIFF_VAR() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("JACOBIAN_DIFF_VAR"));
      then txt;

    case ( txt,
           BackendDAE.SEED_VAR() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("SEED_VAR"));
      then txt;

    case ( txt,
           BackendDAE.OPT_CONSTR() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("OPT_CONSTR"));
      then txt;

    case ( txt,
           BackendDAE.OPT_FCONSTR() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("OPT_FCONSTR"));
      then txt;

    case ( txt,
           BackendDAE.OPT_INPUT_WITH_DER() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("OPT_INPUT_WITH_DER"));
      then txt;

    case ( txt,
           BackendDAE.OPT_INPUT_DER() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("OPT_INPUT_DER"));
      then txt;

    case ( txt,
           BackendDAE.OPT_TGRID() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("OPT_TGRID"));
      then txt;

    case ( txt,
           BackendDAE.OPT_LOOP_INPUT(replaceExp = _) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("OPT_LOOP_INPUT"));
      then txt;

    case ( txt,
           BackendDAE.ALG_STATE() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("ALG_STATE"));
      then txt;

    case ( txt,
           BackendDAE.DAE_RESIDUAL_VAR() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("DAE_RESIDUAL_VAR"));
      then txt;

    case ( txt,
           BackendDAE.DAE_AUX_VAR() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("DAE_AUX_VAR"));
      then txt;

    case ( txt,
           BackendDAE.LOOP_ITERATION() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("LOOP_ITERATION"));
      then txt;

    case ( txt,
           BackendDAE.LOOP_SOLVED() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("LOOP_SOLVED"));
      then txt;

    case ( txt,
           _ )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("#UNKNOWN_VARKIND"));
      then txt;
  end match;
end variabilityString;

public function getAliasVar
  input Tpl.Text in_txt;
  input SimCodeVar.AliasVariable in_a_aliasvar;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_aliasvar)
    local
      Tpl.Text txt;
      DAE.ComponentRef i_varName;

    case ( txt,
           SimCodeVar.NOALIAS() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\"noAlias\""));
      then txt;

    case ( txt,
           SimCodeVar.ALIAS(varName = i_varName) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\"alias\" aliasVariable=\""));
        txt = crefStrNoUnderscore(txt, i_varName);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
      then txt;

    case ( txt,
           SimCodeVar.NEGATEDALIAS(varName = i_varName) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\"negatedAlias\" aliasVariable=\""));
        txt = crefStrNoUnderscore(txt, i_varName);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
      then txt;

    case ( txt,
           _ )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\"noAlias\""));
      then txt;
  end match;
end getAliasVar;

protected function fun_58
  input Tpl.Text in_txt;
  input DAE.Type in_a_type__;
  input String in_a_unit;
  input String in_a_displayUnit;
  input Option<DAE.Exp> in_a_minValue;
  input Option<DAE.Exp> in_a_maxValue;
  input Option<DAE.Exp> in_a_startValue;
  input Option<DAE.Exp> in_a_nominalValue;
  input Boolean in_a_isFixed;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_type__, in_a_unit, in_a_displayUnit, in_a_minValue, in_a_maxValue, in_a_startValue, in_a_nominalValue, in_a_isFixed)
    local
      Tpl.Text txt;
      String a_unit;
      String a_displayUnit;
      Option<DAE.Exp> a_minValue;
      Option<DAE.Exp> a_maxValue;
      Option<DAE.Exp> a_startValue;
      Option<DAE.Exp> a_nominalValue;
      Boolean a_isFixed;
      Absyn.Path i_ci_path;
      DAE.Type i_type__;
      Tpl.Text txt_2;
      String ret_2;
      String ret_1;
      Tpl.Text txt_0;

    case ( txt,
           (i_type__ as DAE.T_INTEGER(varLst = _)),
           a_unit,
           a_displayUnit,
           a_minValue,
           a_maxValue,
           a_startValue,
           _,
           a_isFixed )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("<Integer"));
        txt = ScalarVariableTypeStartAttribute(txt, a_startValue, i_type__);
        txt = ScalarVariableTypeFixedAttribute(txt, a_isFixed);
        txt = ScalarVariableTypeIntegerMinAttribute(txt, a_minValue);
        txt = ScalarVariableTypeIntegerMaxAttribute(txt, a_maxValue);
        txt = ScalarVariableTypeUnitAttribute(txt, a_unit);
        txt = ScalarVariableTypeDisplayUnitAttribute(txt, a_displayUnit);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(" />"));
      then txt;

    case ( txt,
           (i_type__ as DAE.T_REAL(varLst = _)),
           a_unit,
           a_displayUnit,
           a_minValue,
           a_maxValue,
           a_startValue,
           a_nominalValue,
           a_isFixed )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("<Real"));
        txt = ScalarVariableTypeStartAttribute(txt, a_startValue, i_type__);
        txt = ScalarVariableTypeFixedAttribute(txt, a_isFixed);
        txt = ScalarVariableTypeNominalAttribute(txt, a_nominalValue);
        txt = ScalarVariableTypeRealMinAttribute(txt, a_minValue);
        txt = ScalarVariableTypeRealMaxAttribute(txt, a_maxValue);
        txt = ScalarVariableTypeUnitAttribute(txt, a_unit);
        txt = ScalarVariableTypeDisplayUnitAttribute(txt, a_displayUnit);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(" />"));
      then txt;

    case ( txt,
           (i_type__ as DAE.T_BOOL(varLst = _)),
           a_unit,
           a_displayUnit,
           _,
           _,
           a_startValue,
           _,
           a_isFixed )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("<Boolean"));
        txt = ScalarVariableTypeStartAttribute(txt, a_startValue, i_type__);
        txt = ScalarVariableTypeFixedAttribute(txt, a_isFixed);
        txt = ScalarVariableTypeUnitAttribute(txt, a_unit);
        txt = ScalarVariableTypeDisplayUnitAttribute(txt, a_displayUnit);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(" />"));
      then txt;

    case ( txt,
           (i_type__ as DAE.T_STRING(varLst = _)),
           a_unit,
           a_displayUnit,
           _,
           _,
           a_startValue,
           _,
           a_isFixed )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("<String"));
        txt = ScalarVariableTypeStartAttribute(txt, a_startValue, i_type__);
        txt = ScalarVariableTypeFixedAttribute(txt, a_isFixed);
        txt = ScalarVariableTypeUnitAttribute(txt, a_unit);
        txt = ScalarVariableTypeDisplayUnitAttribute(txt, a_displayUnit);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(" />"));
      then txt;

    case ( txt,
           (i_type__ as DAE.T_ENUMERATION(index = _)),
           a_unit,
           a_displayUnit,
           _,
           _,
           a_startValue,
           _,
           a_isFixed )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("<Integer"));
        txt = ScalarVariableTypeStartAttribute(txt, a_startValue, i_type__);
        txt = ScalarVariableTypeFixedAttribute(txt, a_isFixed);
        txt = ScalarVariableTypeUnitAttribute(txt, a_unit);
        txt = ScalarVariableTypeDisplayUnitAttribute(txt, a_displayUnit);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(" />"));
      then txt;

    case ( txt,
           DAE.T_COMPLEX(complexClassType = ClassInf.EXTERNAL_OBJ(path = i_ci_path)),
           _,
           _,
           _,
           _,
           _,
           _,
           _ )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("<ExternalObject path=\""));
        txt_0 = dotPath(Tpl.emptyTxt, i_ci_path);
        ret_1 = Util.escapeModelicaStringToXmlString(Tpl.textString(txt_0));
        txt = Tpl.writeStr(txt, ret_1);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\" />"));
      then txt;

    case ( txt,
           i_type__,
           _,
           _,
           _,
           _,
           _,
           _,
           _ )
      equation
        txt_2 = Tpl.writeTok(Tpl.emptyTxt, Tpl.ST_STRING("ScalarVariableType: "));
        ret_2 = Types.unparseType(i_type__);
        txt_2 = Tpl.writeStr(txt_2, ret_2);
        txt = error(txt, Tpl.sourceInfo("CodegenUtil.tpl", 268, 16), Tpl.textString(txt_2));
      then txt;
  end match;
end fun_58;

public function ScalarVariableType
  input Tpl.Text txt;
  input String a_unit;
  input String a_displayUnit;
  input Option<DAE.Exp> a_minValue;
  input Option<DAE.Exp> a_maxValue;
  input Option<DAE.Exp> a_startValue;
  input Option<DAE.Exp> a_nominalValue;
  input Boolean a_isFixed;
  input DAE.Type a_type__;

  output Tpl.Text out_txt;
algorithm
  out_txt := fun_58(txt, a_type__, a_unit, a_displayUnit, a_minValue, a_maxValue, a_startValue, a_nominalValue, a_isFixed);
end ScalarVariableType;

public function StartString
  input Tpl.Text in_txt;
  input DAE.Exp in_a_exp;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_exp)
    local
      Tpl.Text txt;
      DAE.Exp i_exp;

    case ( txt,
           (i_exp as DAE.ICONST(integer = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("start=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           (i_exp as DAE.RCONST(real = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("start=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           (i_exp as DAE.SCONST(string = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("start=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           (i_exp as DAE.BCONST(bool = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("start=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           (i_exp as DAE.ENUM_LITERAL(name = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("start=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end StartString;

protected function fun_61
  input Tpl.Text in_txt;
  input Option<DAE.Exp> in_a_startValue;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_startValue)
    local
      Tpl.Text txt;
      DAE.Exp i_exp;

    case ( txt,
           SOME(i_exp) )
      equation
        txt = StartString(txt, i_exp);
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end fun_61;

public function ScalarVariableTypeStartAttribute
  input Tpl.Text txt;
  input Option<DAE.Exp> a_startValue;
  input DAE.Type a_type__;

  output Tpl.Text out_txt;
algorithm
  out_txt := fun_61(txt, a_startValue);
end ScalarVariableTypeStartAttribute;

public function ScalarVariableTypeFixedAttribute
  input Tpl.Text txt;
  input Boolean a_isFixed;

  output Tpl.Text out_txt;
algorithm
  out_txt := Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
  out_txt := Tpl.writeTok(out_txt, Tpl.ST_STRING("fixed=\""));
  out_txt := Tpl.writeStr(out_txt, Tpl.booleanString(a_isFixed));
  out_txt := Tpl.writeTok(out_txt, Tpl.ST_STRING("\""));
  out_txt := Tpl.popBlock(out_txt);
end ScalarVariableTypeFixedAttribute;

public function NominalString
  input Tpl.Text in_txt;
  input DAE.Exp in_a_exp;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_exp)
    local
      Tpl.Text txt;
      DAE.Exp i_exp;

    case ( txt,
           (i_exp as DAE.ICONST(integer = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("nominal=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           (i_exp as DAE.RCONST(real = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("nominal=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           (i_exp as DAE.SCONST(string = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("nominal=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           (i_exp as DAE.BCONST(bool = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("nominal=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end NominalString;

public function ScalarVariableTypeNominalAttribute
  input Tpl.Text in_txt;
  input Option<DAE.Exp> in_a_nominalValue;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_nominalValue)
    local
      Tpl.Text txt;
      DAE.Exp i_exp;

    case ( txt,
           SOME(i_exp) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("useNominal=\"true\""));
        txt = NominalString(txt, i_exp);
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           NONE() )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(" useNominal=\"false\""));
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end ScalarVariableTypeNominalAttribute;

public function ScalarVariableTypeUnitAttribute
  input Tpl.Text in_txt;
  input String in_a_unit;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_unit)
    local
      Tpl.Text txt;
      String i_unit;

    case ( txt,
           "" )
      then txt;

    case ( txt,
           i_unit )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("unit=\""));
        txt = Tpl.writeStr(txt, i_unit);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;
  end match;
end ScalarVariableTypeUnitAttribute;

public function ScalarVariableTypeDisplayUnitAttribute
  input Tpl.Text in_txt;
  input String in_a_displayUnit;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_displayUnit)
    local
      Tpl.Text txt;
      String i_displayUnit;

    case ( txt,
           "" )
      then txt;

    case ( txt,
           i_displayUnit )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("displayUnit=\""));
        txt = Tpl.writeStr(txt, i_displayUnit);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;
  end match;
end ScalarVariableTypeDisplayUnitAttribute;

public function MinString
  input Tpl.Text in_txt;
  input DAE.Exp in_a_exp;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_exp)
    local
      Tpl.Text txt;
      DAE.Exp i_exp;

    case ( txt,
           (i_exp as DAE.ICONST(integer = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("min=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           (i_exp as DAE.RCONST(real = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("min=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           (i_exp as DAE.SCONST(string = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("min=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           (i_exp as DAE.BCONST(bool = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("min=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           (i_exp as DAE.ENUM_LITERAL(name = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("min=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end MinString;

public function ScalarVariableTypeIntegerMinAttribute
  input Tpl.Text in_txt;
  input Option<DAE.Exp> in_a_minValue;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_minValue)
    local
      Tpl.Text txt;
      DAE.Exp i_exp;

    case ( txt,
           SOME(i_exp) )
      equation
        txt = MinString(txt, i_exp);
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end ScalarVariableTypeIntegerMinAttribute;

public function MaxString
  input Tpl.Text in_txt;
  input DAE.Exp in_a_exp;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_exp)
    local
      Tpl.Text txt;
      DAE.Exp i_exp;

    case ( txt,
           (i_exp as DAE.ICONST(integer = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("max=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           (i_exp as DAE.RCONST(real = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("max=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           (i_exp as DAE.SCONST(string = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("max=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           (i_exp as DAE.BCONST(bool = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("max=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           (i_exp as DAE.ENUM_LITERAL(name = _)) )
      equation
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("max=\""));
        txt = initValXml(txt, i_exp);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.popBlock(txt);
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end MaxString;

public function ScalarVariableTypeIntegerMaxAttribute
  input Tpl.Text in_txt;
  input Option<DAE.Exp> in_a_maxValue;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_maxValue)
    local
      Tpl.Text txt;
      DAE.Exp i_exp;

    case ( txt,
           SOME(i_exp) )
      equation
        txt = MaxString(txt, i_exp);
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end ScalarVariableTypeIntegerMaxAttribute;

public function ScalarVariableTypeRealMinAttribute
  input Tpl.Text in_txt;
  input Option<DAE.Exp> in_a_minValue;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_minValue)
    local
      Tpl.Text txt;
      DAE.Exp i_exp;

    case ( txt,
           SOME(i_exp) )
      equation
        txt = MinString(txt, i_exp);
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end ScalarVariableTypeRealMinAttribute;

public function ScalarVariableTypeRealMaxAttribute
  input Tpl.Text in_txt;
  input Option<DAE.Exp> in_a_maxValue;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_maxValue)
    local
      Tpl.Text txt;
      DAE.Exp i_exp;

    case ( txt,
           SOME(i_exp) )
      equation
        txt = MaxString(txt, i_exp);
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end ScalarVariableTypeRealMaxAttribute;

public function dotPath
  input Tpl.Text in_txt;
  input Absyn.Path in_a_path;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_path)
    local
      Tpl.Text txt;
      String i_name_1;
      Absyn.Path i_path;
      Absyn.Ident i_name;

    case ( txt,
           Absyn.QUALIFIED(name = i_name, path = i_path) )
      equation
        txt = Tpl.writeStr(txt, i_name);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("."));
        txt = dotPath(txt, i_path);
      then txt;

    case ( txt,
           Absyn.IDENT(name = i_name_1) )
      equation
        txt = Tpl.writeStr(txt, i_name_1);
      then txt;

    case ( txt,
           Absyn.FULLYQUALIFIED(path = i_path) )
      equation
        txt = dotPath(txt, i_path);
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end dotPath;

public function error
  input Tpl.Text txt;
  input SourceInfo a_srcInfo;
  input String a_errMessage;

  output Tpl.Text out_txt;
protected
  String ret_0;
algorithm
  Tpl.addSourceTemplateError(a_errMessage, a_srcInfo);
  out_txt := Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                   "\n",
                                   "#error \""
                               }, false));
  ret_0 := Error.infoStr(a_srcInfo);
  out_txt := Tpl.writeStr(out_txt, ret_0);
  out_txt := Tpl.writeTok(out_txt, Tpl.ST_STRING(" "));
  out_txt := Tpl.writeStr(out_txt, a_errMessage);
  out_txt := Tpl.writeTok(out_txt, Tpl.ST_STRING("\""));
  out_txt := Tpl.writeTok(out_txt, Tpl.ST_NEW_LINE());
end error;

public function errorMsg
  input Tpl.Text txt;
  input String a_errMessage;

  output Tpl.Text out_txt;
algorithm
  Tpl.addTemplateError(a_errMessage);
  out_txt := Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                   "\n",
                                   "#error \""
                               }, false));
  out_txt := Tpl.writeStr(out_txt, a_errMessage);
  out_txt := Tpl.writeTok(out_txt, Tpl.ST_STRING("\""));
  out_txt := Tpl.writeTok(out_txt, Tpl.ST_NEW_LINE());
end errorMsg;

annotation(__OpenModelica_Interface="backend");
end CodegenUtil;