encapsulated package CodegenUtilSimulation
"
  file:        CodegenUtilSimulation.mo
  package:     CodegenUtilSimulation
  description: Generated by Susan.
"

public import Tpl;

public import SimCodeVar;
public import SimCode;
public import BackendDAE;
public import System;
public import DAE;
public import Util;
public import List;
public import Expression;
public import ExpressionDump;
public import DAEDump;
public import ExpressionDumpTpl;
public import CodegenUtil;

public function modelNamePrefix
  input Tpl.Text in_txt;
  input SimCode.SimCode in_a_simCode;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_simCode)
    local
      Tpl.Text txt;
      String i_fileNamePrefix;
      String ret_0;

    case ( txt,
           SimCode.SIMCODE(fileNamePrefix = i_fileNamePrefix) )
      equation
        ret_0 = System.makeC89Identifier(i_fileNamePrefix);
        txt = Tpl.writeStr(txt, ret_0);
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end modelNamePrefix;

public function fileNamePrefix
  input Tpl.Text in_txt;
  input SimCode.SimCode in_a_simCode;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_simCode)
    local
      Tpl.Text txt;
      String i_fileNamePrefix;

    case ( txt,
           SimCode.SIMCODE(fileNamePrefix = i_fileNamePrefix) )
      equation
        txt = Tpl.writeStr(txt, i_fileNamePrefix);
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end fileNamePrefix;

public function fullPathPrefix
  input Tpl.Text in_txt;
  input SimCode.SimCode in_a_simCode;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_simCode)
    local
      Tpl.Text txt;
      String i_fullPathPrefix;

    case ( txt,
           SimCode.SIMCODE(fullPathPrefix = i_fullPathPrefix) )
      equation
        txt = Tpl.writeStr(txt, i_fullPathPrefix);
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end fullPathPrefix;

public function equationIndex
  input Tpl.Text in_txt;
  input SimCode.SimEqSystem in_a_eq;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_eq)
    local
      Tpl.Text txt;
      Integer i_aliasOf;
      Integer i_nls_index;
      Integer i_ls_index;
      Integer i_index;

    case ( txt,
           SimCode.SES_RESIDUAL(index = i_index) )
      equation
        txt = Tpl.writeStr(txt, intString(i_index));
      then txt;

    case ( txt,
           SimCode.SES_SIMPLE_ASSIGN(index = i_index) )
      equation
        txt = Tpl.writeStr(txt, intString(i_index));
      then txt;

    case ( txt,
           SimCode.SES_SIMPLE_ASSIGN_CONSTRAINTS(index = i_index) )
      equation
        txt = Tpl.writeStr(txt, intString(i_index));
      then txt;

    case ( txt,
           SimCode.SES_ARRAY_CALL_ASSIGN(index = i_index) )
      equation
        txt = Tpl.writeStr(txt, intString(i_index));
      then txt;

    case ( txt,
           SimCode.SES_IFEQUATION(index = i_index) )
      equation
        txt = Tpl.writeStr(txt, intString(i_index));
      then txt;

    case ( txt,
           SimCode.SES_ALGORITHM(index = i_index) )
      equation
        txt = Tpl.writeStr(txt, intString(i_index));
      then txt;

    case ( txt,
           SimCode.SES_INVERSE_ALGORITHM(index = i_index) )
      equation
        txt = Tpl.writeStr(txt, intString(i_index));
      then txt;

    case ( txt,
           SimCode.SES_LINEAR(lSystem = SimCode.LINEARSYSTEM(index = i_ls_index)) )
      equation
        txt = Tpl.writeStr(txt, intString(i_ls_index));
      then txt;

    case ( txt,
           SimCode.SES_NONLINEAR(nlSystem = SimCode.NONLINEARSYSTEM(index = i_nls_index)) )
      equation
        txt = Tpl.writeStr(txt, intString(i_nls_index));
      then txt;

    case ( txt,
           SimCode.SES_MIXED(index = i_index) )
      equation
        txt = Tpl.writeStr(txt, intString(i_index));
      then txt;

    case ( txt,
           SimCode.SES_WHEN(index = i_index) )
      equation
        txt = Tpl.writeStr(txt, intString(i_index));
      then txt;

    case ( txt,
           SimCode.SES_FOR_LOOP(index = i_index) )
      equation
        txt = Tpl.writeStr(txt, intString(i_index));
      then txt;

    case ( txt,
           SimCode.SES_ALIAS(aliasOf = i_aliasOf) )
      equation
        txt = Tpl.writeStr(txt, intString(i_aliasOf));
      then txt;

    case ( txt,
           SimCode.SES_ALGEBRAIC_SYSTEM(index = i_index) )
      equation
        txt = Tpl.writeStr(txt, intString(i_index));
      then txt;

    case ( txt,
           _ )
      equation
        txt = CodegenUtil.error(txt, Tpl.sourceInfo("CodegenUtilSimulation.tpl", 97, 14), "equationIndex failed");
      then txt;
  end match;
end equationIndex;

public function equationIndexAlternativeTearing
  input Tpl.Text in_txt;
  input SimCode.SimEqSystem in_a_eq;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_eq)
    local
      Tpl.Text txt;
      Integer i_at_index;

    case ( txt,
           SimCode.SES_LINEAR(alternativeTearing = SOME(SimCode.LINEARSYSTEM(index = i_at_index))) )
      equation
        txt = Tpl.writeStr(txt, intString(i_at_index));
      then txt;

    case ( txt,
           SimCode.SES_NONLINEAR(alternativeTearing = SOME(SimCode.NONLINEARSYSTEM(index = i_at_index))) )
      equation
        txt = Tpl.writeStr(txt, intString(i_at_index));
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end equationIndexAlternativeTearing;

protected function lm_45
  input Tpl.Text in_txt;
  input list<DAE.Statement> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<DAE.Statement> rest;
      DAE.Statement i_stmt;
      String ret_0;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_stmt :: rest )
      equation
        ret_0 = DAEDump.ppStmtStr(i_stmt, 2);
        txt = CodegenUtil.escapeCComments(txt, ret_0);
        txt = lm_45(txt, rest);
      then txt;
  end match;
end lm_45;

protected function lm_46
  input Tpl.Text in_txt;
  input list<DAE.Statement> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<DAE.Statement> rest;
      DAE.Statement i_stmt;
      String ret_0;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_stmt :: rest )
      equation
        ret_0 = DAEDump.ppStmtStr(i_stmt, 2);
        txt = CodegenUtil.escapeCComments(txt, ret_0);
        txt = lm_46(txt, rest);
      then txt;
  end match;
end lm_46;

protected function lm_47
  input Tpl.Text in_txt;
  input list<SimCodeVar.SimVar> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<SimCodeVar.SimVar> rest;
      DAE.ComponentRef i_cr;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           SimCodeVar.SIMVAR(name = i_cr) :: rest )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("<var>"));
        txt = ExpressionDumpTpl.dumpCref(txt, i_cr);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("</var>"));
        txt = Tpl.nextIter(txt);
        txt = lm_47(txt, rest);
      then txt;

    case ( txt,
           _ :: rest )
      equation
        txt = lm_47(txt, rest);
      then txt;
  end match;
end lm_47;

protected function lm_48
  input Tpl.Text in_txt;
  input list<DAE.Exp> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<DAE.Exp> rest;
      DAE.Exp i_exp;
      Tpl.Text txt_0;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_exp :: rest )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("<cell>"));
        txt_0 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_exp, "\"");
        txt = CodegenUtil.escapeCComments(txt, Tpl.textString(txt_0));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("</cell>"));
        txt = Tpl.nextIter(txt);
        txt = lm_48(txt, rest);
      then txt;
  end match;
end lm_48;

protected function fun_49
  input Tpl.Text in_txt;
  input SimCode.SimEqSystem in_a_eq;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_eq)
    local
      Tpl.Text txt;
      DAE.Exp i_e_exp;
      Tpl.Text txt_0;

    case ( txt,
           SimCode.SES_RESIDUAL(exp = i_e_exp) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("<residual>"));
        txt_0 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_exp, "\"");
        txt = CodegenUtil.escapeCComments(txt, Tpl.textString(txt_0));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("</residual>"));
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end fun_49;

protected function lm_50
  input Tpl.Text in_txt;
  input list<tuple<Integer, Integer, SimCode.SimEqSystem>> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<tuple<Integer, Integer, SimCode.SimEqSystem>> rest;
      SimCode.SimEqSystem i_eq;
      Integer i_i2;
      Integer i_i1;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           (i_i1, i_i2, i_eq) :: rest )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("<cell row=\""));
        txt = Tpl.writeStr(txt, intString(i_i1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\" col=\""));
        txt = Tpl.writeStr(txt, intString(i_i2));
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("\">\n"));
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(2));
        txt = fun_49(txt, i_eq);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.popBlock(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("</cell>"));
        txt = lm_50(txt, rest);
      then txt;
  end match;
end lm_50;

protected function lm_51
  input Tpl.Text in_txt;
  input list<DAE.ComponentRef> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<DAE.ComponentRef> rest;
      DAE.ComponentRef i_cr;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_cr :: rest )
      equation
        txt = ExpressionDumpTpl.dumpCref(txt, i_cr);
        txt = Tpl.nextIter(txt);
        txt = lm_51(txt, rest);
      then txt;
  end match;
end lm_51;

protected function lm_52
  input Tpl.Text in_txt;
  input list<SimCode.SimEqSystem> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<SimCode.SimEqSystem> rest;
      SimCode.SimEqSystem i_eq;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_eq :: rest )
      equation
        txt = equationIndex(txt, i_eq);
        txt = Tpl.nextIter(txt);
        txt = lm_52(txt, rest);
      then txt;
  end match;
end lm_52;

protected function lm_53
  input Tpl.Text in_txt;
  input list<SimCodeVar.SimVar> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<SimCodeVar.SimVar> rest;
      DAE.ComponentRef i_cr;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           SimCodeVar.SIMVAR(name = i_cr) :: rest )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("<var>"));
        txt = ExpressionDumpTpl.dumpCref(txt, i_cr);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("</var>"));
        txt = Tpl.nextIter(txt);
        txt = lm_53(txt, rest);
      then txt;

    case ( txt,
           _ :: rest )
      equation
        txt = lm_53(txt, rest);
      then txt;
  end match;
end lm_53;

protected function lm_54
  input Tpl.Text in_txt;
  input list<SimCode.SimEqSystem> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<SimCode.SimEqSystem> rest;
      SimCode.SimEqSystem i_eq;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_eq :: rest )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("<discrete index=\""));
        txt = equationIndex(txt, i_eq);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\" />"));
        txt = lm_54(txt, rest);
      then txt;
  end match;
end lm_54;

protected function lm_55
  input Tpl.Text in_txt;
  input list<SimCode.SimEqSystem> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<SimCode.SimEqSystem> rest;
      SimCode.SimEqSystem i_eq;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_eq :: rest )
      equation
        txt = equationIndex(txt, i_eq);
        txt = Tpl.nextIter(txt);
        txt = lm_55(txt, rest);
      then txt;
  end match;
end lm_55;

protected function lm_56
  input Tpl.Text in_txt;
  input list<DAE.ComponentRef> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<DAE.ComponentRef> rest;
      DAE.ComponentRef i_cond;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_cond :: rest )
      equation
        txt = ExpressionDumpTpl.dumpCref(txt, i_cond);
        txt = Tpl.nextIter(txt);
        txt = lm_56(txt, rest);
      then txt;
  end match;
end lm_56;

protected function lm_57
  input Tpl.Text in_txt;
  input list<tuple<DAE.Exp, list<SimCode.SimEqSystem>>> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<tuple<DAE.Exp, list<SimCode.SimEqSystem>>> rest;
      list<SimCode.SimEqSystem> i_eqs;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           (_, i_eqs) :: rest )
      equation
        txt = dumpEqs(txt, i_eqs);
        txt = lm_57(txt, rest);
      then txt;
  end match;
end lm_57;

protected function fun_58
  input Tpl.Text in_txt;
  input SimCode.SimEqSystem in_a_eq;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_eq)
    local
      Tpl.Text txt;
      DAE.Exp i_e_endIt;
      DAE.Exp i_e_startIt;
      DAE.Exp i_e_iter;
      list<SimCode.SimEqSystem> i_elsebranch;
      list<tuple<DAE.Exp, list<SimCode.SimEqSystem>>> i_ifbranches;
      list<DAE.ComponentRef> i_conditions;
      list<BackendDAE.WhenOperator> i_whenStmtLst;
      list<SimCode.SimEqSystem> i_residual_equations;
      Boolean i_e_linearSystem;
      Option<SimCode.DerivativeMatrix> i_matrix;
      list<SimCodeVar.SimVar> i_e_discVars;
      list<SimCode.SimEqSystem> i_e_discEqs;
      SimCode.SimEqSystem i_e_cont;
      list<SimCode.SimEqSystem> i_nls_eqs;
      list<DAE.ComponentRef> i_nls_crefs;
      Integer i_nls_indexNonLinearSystem;
      list<tuple<Integer, Integer, SimCode.SimEqSystem>> i_ls_simJac;
      list<DAE.Exp> i_ls_beqs;
      list<SimCodeVar.SimVar> i_ls_vars;
      list<DAE.Statement> i_e_statements;
      DAE.ComponentRef i_lhs_componentRef;
      BackendDAE.Constraints i_e_cons;
      DAE.ComponentRef i_e_cref;
      DAE.Exp i_e_exp;
      SimCode.SimEqSystem i_e;
      Tpl.Text txt_15;
      Tpl.Text txt_14;
      Tpl.Text txt_13;
      Tpl.Text txt_12;
      Tpl.Text l_forstatement;
      Tpl.Text l_elsebr;
      Tpl.Text l_branches;
      Tpl.Text l_body;
      Integer ret_7;
      Tpl.Text l_detailedDescription;
      list<SimCode.SimEqSystem> ret_5;
      Tpl.Text txt_4;
      Tpl.Text txt_3;
      Tpl.Text txt_2;
      Tpl.Text txt_1;
      Tpl.Text txt_0;

    case ( txt,
           (i_e as SimCode.SES_RESIDUAL(exp = i_e_exp)) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("equation index: "));
        txt = equationIndex(txt, i_e);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("type: RESIDUAL\n"));
        txt_0 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_exp, "\"");
        txt = CodegenUtil.escapeCComments(txt, Tpl.textString(txt_0));
      then txt;

    case ( txt,
           (i_e as SimCode.SES_SIMPLE_ASSIGN(cref = i_e_cref, exp = i_e_exp)) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("equation index: "));
        txt = equationIndex(txt, i_e);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("type: SIMPLE_ASSIGN\n"));
        txt = ExpressionDumpTpl.dumpCref(txt, i_e_cref);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(" = "));
        txt_1 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_exp, "\"");
        txt = CodegenUtil.escapeCComments(txt, Tpl.textString(txt_1));
      then txt;

    case ( txt,
           (i_e as SimCode.SES_SIMPLE_ASSIGN_CONSTRAINTS(cref = i_e_cref, exp = i_e_exp, cons = i_e_cons)) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("equation index: "));
        txt = equationIndex(txt, i_e);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("type: SIMPLE_ASSIGN_CONSTRAINTS\n"));
        txt = ExpressionDumpTpl.dumpCref(txt, i_e_cref);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(" = "));
        txt_2 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_exp, "\"");
        txt = CodegenUtil.escapeCComments(txt, Tpl.textString(txt_2));
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("constraints: "));
        txt_3 = ExpressionDumpTpl.dumpConstraints(Tpl.emptyTxt, i_e_cons);
        txt = CodegenUtil.escapeCComments(txt, Tpl.textString(txt_3));
      then txt;

    case ( txt,
           (i_e as SimCode.SES_ARRAY_CALL_ASSIGN(lhs = DAE.CREF(componentRef = i_lhs_componentRef), exp = i_e_exp)) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("equation index: "));
        txt = equationIndex(txt, i_e);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "type: ARRAY_CALL_ASSIGN\n",
                                    "\n"
                                }, true));
        txt = ExpressionDumpTpl.dumpCref(txt, i_lhs_componentRef);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(" = "));
        txt_4 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_exp, "\"");
        txt = CodegenUtil.escapeCComments(txt, Tpl.textString(txt_4));
      then txt;

    case ( txt,
           SimCode.SES_ALGORITHM(statements = {}) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("empty algorithm"));
      then txt;

    case ( txt,
           (i_e as SimCode.SES_ALGORITHM(statements = (i_e_statements as _ :: _))) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("equation index: "));
        txt = equationIndex(txt, i_e);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "type: ALGORITHM\n",
                                    "\n"
                                }, true));
        txt = lm_45(txt, i_e_statements);
      then txt;

    case ( txt,
           (i_e as SimCode.SES_INVERSE_ALGORITHM(statements = (i_e_statements as _ :: _))) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("equation index: "));
        txt = equationIndex(txt, i_e);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "type: INVERSE ALGORITHM\n",
                                    "\n"
                                }, true));
        txt = lm_46(txt, i_e_statements);
      then txt;

    case ( txt,
           (i_e as SimCode.SES_LINEAR(lSystem = SimCode.LINEARSYSTEM(vars = i_ls_vars, beqs = i_ls_beqs, simJac = i_ls_simJac))) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("equation index: "));
        txt = equationIndex(txt, i_e);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "type: LINEAR\n",
                                    "\n"
                                }, true));
        txt = Tpl.pushIter(txt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_NEW_LINE()), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        txt = lm_47(txt, i_ls_vars);
        txt = Tpl.popIter(txt);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("<row>\n"));
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(2));
        txt = Tpl.pushIter(txt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_NEW_LINE()), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        txt = lm_48(txt, i_ls_beqs);
        txt = Tpl.popIter(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_NEW_LINE());
        txt = Tpl.popBlock(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "</row>\n",
                                    "<matrix>\n"
                                }, true));
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(2));
        txt = lm_50(txt, i_ls_simJac);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.popBlock(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("</matrix>"));
      then txt;

    case ( txt,
           (i_e as SimCode.SES_NONLINEAR(nlSystem = SimCode.NONLINEARSYSTEM(indexNonLinearSystem = i_nls_indexNonLinearSystem, crefs = i_nls_crefs, eqs = i_nls_eqs))) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("equation index: "));
        txt = equationIndex(txt, i_e);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("indexNonlinear: "));
        txt = Tpl.writeStr(txt, intString(i_nls_indexNonLinearSystem));
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "type: NONLINEAR\n",
                                    "\n",
                                    "vars: {"
                                }, false));
        txt = Tpl.pushIter(txt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_STRING(", ")), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        txt = lm_51(txt, i_nls_crefs);
        txt = Tpl.popIter(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "}\n",
                                    "eqns: {"
                                }, false));
        txt = Tpl.pushIter(txt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_STRING(", ")), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        txt = lm_52(txt, i_nls_eqs);
        txt = Tpl.popIter(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("}"));
      then txt;

    case ( txt,
           (i_e as SimCode.SES_MIXED(cont = i_e_cont, discEqs = i_e_discEqs, discVars = i_e_discVars)) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("equation index: "));
        txt = equationIndex(txt, i_e);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "type: MIXED\n",
                                    "\n"
                                }, true));
        ret_5 = List.fill(i_e_cont, 1);
        txt = dumpEqs(txt, ret_5);
        txt = Tpl.softNewLine(txt);
        txt = dumpEqs(txt, i_e_discEqs);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "\n",
                                    "\n",
                                    "<mixed>\n"
                                }, true));
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(2));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("<continuous index=\""));
        txt = equationIndex(txt, i_e_cont);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("\" />\n"));
        txt = Tpl.pushIter(txt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_STRING(",")), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        txt = lm_53(txt, i_e_discVars);
        txt = Tpl.popIter(txt);
        txt = Tpl.softNewLine(txt);
        txt = lm_54(txt, i_e_discEqs);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.popBlock(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("</mixed>"));
      then txt;

    case ( txt,
           (i_e as SimCode.SES_ALGEBRAIC_SYSTEM(residual = SimCode.OMSI_FUNCTION(equations = i_residual_equations), matrix = i_matrix, linearSystem = i_e_linearSystem)) )
      equation
        l_detailedDescription = dumpAlgSystemOps(Tpl.emptyTxt, i_matrix);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("equation index: "));
        txt = equationIndex(txt, i_e);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "type: ALGEBRAIC_SYSTEM\n",
                                    "is linear: "
                                }, false));
        txt = Tpl.writeStr(txt, Tpl.booleanString(i_e_linearSystem));
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("depending functions indices: "));
        txt = Tpl.pushIter(txt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_STRING(", ")), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        txt = lm_55(txt, i_residual_equations);
        txt = Tpl.popIter(txt);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("dimension: "));
        ret_7 = listLength(i_residual_equations);
        txt = Tpl.writeStr(txt, intString(ret_7));
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeText(txt, l_detailedDescription);
      then txt;

    case ( txt,
           (i_e as SimCode.SES_WHEN(whenStmtLst = i_whenStmtLst, conditions = i_conditions)) )
      equation
        l_body = dumpWhenOps(Tpl.emptyTxt, i_whenStmtLst);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("equation index: "));
        txt = equationIndex(txt, i_e);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "type: WHEN\n",
                                    "\n",
                                    "when {"
                                }, false));
        txt = Tpl.pushIter(txt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_STRING(", ")), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        txt = lm_56(txt, i_conditions);
        txt = Tpl.popIter(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("} then\n"));
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(2));
        txt = Tpl.writeText(txt, l_body);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.popBlock(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("end when;"));
      then txt;

    case ( txt,
           (i_e as SimCode.SES_IFEQUATION(ifbranches = i_ifbranches, elsebranch = i_elsebranch)) )
      equation
        l_branches = lm_57(Tpl.emptyTxt, i_ifbranches);
        l_elsebr = dumpEqs(Tpl.emptyTxt, i_elsebranch);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("equation index: "));
        txt = equationIndex(txt, i_e);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "type: IFEQUATION\n",
                                    "\n"
                                }, true));
        txt = Tpl.writeText(txt, l_branches);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeText(txt, l_elsebr);
      then txt;

    case ( txt,
           (i_e as SimCode.SES_FOR_LOOP(iter = i_e_iter, startIt = i_e_startIt, endIt = i_e_endIt, cref = i_e_cref, exp = i_e_exp)) )
      equation
        l_forstatement = Tpl.emptyTxt;
        l_forstatement = Tpl.writeTok(l_forstatement, Tpl.ST_STRING("for "));
        txt_12 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_iter, "\"");
        l_forstatement = CodegenUtil.escapeCComments(l_forstatement, Tpl.textString(txt_12));
        l_forstatement = Tpl.writeTok(l_forstatement, Tpl.ST_STRING(" in "));
        txt_13 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_startIt, "\"");
        l_forstatement = CodegenUtil.escapeCComments(l_forstatement, Tpl.textString(txt_13));
        l_forstatement = Tpl.writeTok(l_forstatement, Tpl.ST_STRING(" : "));
        txt_14 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_endIt, "\"");
        l_forstatement = CodegenUtil.escapeCComments(l_forstatement, Tpl.textString(txt_14));
        l_forstatement = Tpl.pushBlock(l_forstatement, Tpl.BT_INDENT(1));
        l_forstatement = Tpl.writeTok(l_forstatement, Tpl.ST_STRING("loop"));
        l_forstatement = Tpl.writeTok(l_forstatement, Tpl.ST_NEW_LINE());
        l_forstatement = Tpl.popBlock(l_forstatement);
        l_forstatement = Tpl.pushBlock(l_forstatement, Tpl.BT_INDENT(2));
        l_forstatement = ExpressionDumpTpl.dumpCref(l_forstatement, i_e_cref);
        l_forstatement = Tpl.writeTok(l_forstatement, Tpl.ST_STRING(" = "));
        txt_15 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_exp, "\"");
        l_forstatement = CodegenUtil.escapeCComments(l_forstatement, Tpl.textString(txt_15));
        l_forstatement = Tpl.writeTok(l_forstatement, Tpl.ST_STRING("; "));
        l_forstatement = Tpl.popBlock(l_forstatement);
        l_forstatement = Tpl.writeTok(l_forstatement, Tpl.ST_STRING("end for"));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("equation index: "));
        txt = equationIndex(txt, i_e);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("type: FOR_LOOP\n"));
        txt = Tpl.writeText(txt, l_forstatement);
      then txt;

    case ( txt,
           _ )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("unknown equation"));
      then txt;
  end match;
end fun_58;

protected function lm_59
  input Tpl.Text in_txt;
  input list<SimCode.SimEqSystem> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<SimCode.SimEqSystem> rest;
      SimCode.SimEqSystem i_eq;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_eq :: rest )
      equation
        txt = fun_58(txt, i_eq);
        txt = lm_59(txt, rest);
      then txt;
  end match;
end lm_59;

public function dumpEqs
  input Tpl.Text txt;
  input list<SimCode.SimEqSystem> a_eqs;

  output Tpl.Text out_txt;
algorithm
  out_txt := lm_59(txt, a_eqs);
end dumpEqs;

protected function lm_61
  input Tpl.Text in_txt;
  input list<SimCode.OMSIFunction> in_items;
  input Tpl.Text in_a_varsBuffer;
  input Tpl.Text in_a_columnBuffer;

  output Tpl.Text out_txt;
  output Tpl.Text out_a_varsBuffer;
  output Tpl.Text out_a_columnBuffer;
algorithm
  (out_txt, out_a_varsBuffer, out_a_columnBuffer) :=
  match(in_txt, in_items, in_a_varsBuffer, in_a_columnBuffer)
    local
      Tpl.Text txt;
      list<SimCode.OMSIFunction> rest;
      Tpl.Text a_varsBuffer;
      Tpl.Text a_columnBuffer;
      SimCode.OMSIFunction i_column;

    case ( txt,
           {},
           a_varsBuffer,
           a_columnBuffer )
      then (txt, a_varsBuffer, a_columnBuffer);

    case ( txt,
           i_column :: rest,
           a_varsBuffer,
           a_columnBuffer )
      equation
        (txt, a_columnBuffer, a_varsBuffer) = dumpAlgSystemColumn(txt, i_column, a_columnBuffer, a_varsBuffer);
        (txt, a_varsBuffer, a_columnBuffer) = lm_61(txt, rest, a_varsBuffer, a_columnBuffer);
      then (txt, a_varsBuffer, a_columnBuffer);
  end match;
end lm_61;

protected function fun_62
  input Tpl.Text in_txt;
  input Option<SimCode.DerivativeMatrix> in_a_derivativeMatrix;
  input Tpl.Text in_a_varsBuffer;
  input Tpl.Text in_a_columnBuffer;

  output Tpl.Text out_txt;
  output Tpl.Text out_a_varsBuffer;
  output Tpl.Text out_a_columnBuffer;
algorithm
  (out_txt, out_a_varsBuffer, out_a_columnBuffer) :=
  match(in_txt, in_a_derivativeMatrix, in_a_varsBuffer, in_a_columnBuffer)
    local
      Tpl.Text txt;
      Tpl.Text a_varsBuffer;
      Tpl.Text a_columnBuffer;
      list<SimCode.OMSIFunction> i_matrix_columns;
      Tpl.Text l_0__;

    case ( txt,
           SOME(SimCode.DERIVATIVE_MATRIX(columns = i_matrix_columns)),
           a_varsBuffer,
           a_columnBuffer )
      equation
        (l_0__, a_varsBuffer, a_columnBuffer) = lm_61(Tpl.emptyTxt, i_matrix_columns, a_varsBuffer, a_columnBuffer);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("iteration vars: "));
        txt = Tpl.writeText(txt, a_varsBuffer);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_NEW_LINE());
        txt = Tpl.writeText(txt, a_columnBuffer);
      then (txt, a_varsBuffer, a_columnBuffer);

    case ( txt,
           _,
           a_varsBuffer,
           a_columnBuffer )
      then (txt, a_varsBuffer, a_columnBuffer);
  end match;
end fun_62;

public function dumpAlgSystemOps
  input Tpl.Text txt;
  input Option<SimCode.DerivativeMatrix> a_derivativeMatrix;

  output Tpl.Text out_txt;
protected
  Tpl.Text l_columnBuffer;
  Tpl.Text l_varsBuffer;
algorithm
  l_varsBuffer := Tpl.emptyTxt;
  l_columnBuffer := Tpl.emptyTxt;
  (out_txt, l_varsBuffer, l_columnBuffer) := fun_62(txt, a_derivativeMatrix, l_varsBuffer, l_columnBuffer);
end dumpAlgSystemOps;

protected function lm_64
  input Tpl.Text in_txt;
  input list<SimCodeVar.SimVar> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<SimCodeVar.SimVar> rest;
      DAE.ComponentRef i_name;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           SimCodeVar.SIMVAR(name = i_name) :: rest )
      equation
        txt = ExpressionDumpTpl.dumpCref(txt, i_name);
        txt = Tpl.nextIter(txt);
        txt = lm_64(txt, rest);
      then txt;

    case ( txt,
           _ :: rest )
      equation
        txt = lm_64(txt, rest);
      then txt;
  end match;
end lm_64;

protected function lm_65
  input Tpl.Text in_txt;
  input list<SimCode.SimEqSystem> in_items;
  input Tpl.Text in_a_columnBuffer;

  output Tpl.Text out_txt;
  output Tpl.Text out_a_columnBuffer;
algorithm
  (out_txt, out_a_columnBuffer) :=
  match(in_txt, in_items, in_a_columnBuffer)
    local
      Tpl.Text txt;
      list<SimCode.SimEqSystem> rest;
      Tpl.Text a_columnBuffer;
      DAE.Exp i_equation_exp;
      DAE.ComponentRef i_equation_cref;
      Tpl.Text txt_0;

    case ( txt,
           {},
           a_columnBuffer )
      then (txt, a_columnBuffer);

    case ( txt,
           SimCode.SES_SIMPLE_ASSIGN(cref = i_equation_cref, exp = i_equation_exp) :: rest,
           a_columnBuffer )
      equation
        a_columnBuffer = ExpressionDumpTpl.dumpCref(a_columnBuffer, i_equation_cref);
        a_columnBuffer = Tpl.writeTok(a_columnBuffer, Tpl.ST_STRING(" = "));
        txt_0 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_equation_exp, "\"");
        a_columnBuffer = CodegenUtil.escapeCComments(a_columnBuffer, Tpl.textString(txt_0));
        a_columnBuffer = Tpl.writeTok(a_columnBuffer, Tpl.ST_NEW_LINE());
        (txt, a_columnBuffer) = lm_65(txt, rest, a_columnBuffer);
      then (txt, a_columnBuffer);

    case ( txt,
           _ :: rest,
           a_columnBuffer )
      equation
        (txt, a_columnBuffer) = lm_65(txt, rest, a_columnBuffer);
      then (txt, a_columnBuffer);
  end match;
end lm_65;

public function dumpAlgSystemColumn
  input Tpl.Text in_txt;
  input SimCode.OMSIFunction in_a_column;
  input Tpl.Text in_a_columnBuffer;
  input Tpl.Text in_a_varsBuffer;

  output Tpl.Text out_txt;
  output Tpl.Text out_a_columnBuffer;
  output Tpl.Text out_a_varsBuffer;
algorithm
  (out_txt, out_a_columnBuffer, out_a_varsBuffer) :=
  match(in_txt, in_a_column, in_a_columnBuffer, in_a_varsBuffer)
    local
      Tpl.Text txt;
      Tpl.Text a_columnBuffer;
      Tpl.Text a_varsBuffer;
      list<SimCode.SimEqSystem> i_equations;
      list<SimCodeVar.SimVar> i_inputVars;
      Tpl.Text l_0__;

    case ( txt,
           SimCode.OMSI_FUNCTION(inputVars = i_inputVars, equations = i_equations),
           a_columnBuffer,
           a_varsBuffer )
      equation
        a_varsBuffer = Tpl.pushIter(a_varsBuffer, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_STRING(", ")), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        a_varsBuffer = lm_64(a_varsBuffer, i_inputVars);
        a_varsBuffer = Tpl.popIter(a_varsBuffer);
        (l_0__, a_columnBuffer) = lm_65(Tpl.emptyTxt, i_equations, a_columnBuffer);
      then (txt, a_columnBuffer, a_varsBuffer);

    case ( txt,
           _,
           a_columnBuffer,
           a_varsBuffer )
      then (txt, a_columnBuffer, a_varsBuffer);
  end match;
end dumpAlgSystemColumn;

public function dumpWhenOps
  input Tpl.Text in_txt;
  input list<BackendDAE.WhenOperator> in_a_whenOps;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_whenOps)
    local
      Tpl.Text txt;
      DAE.Exp i_e_exp;
      DAE.Exp i_e_level;
      DAE.Exp i_e_message;
      DAE.Exp i_e_condition;
      DAE.Exp i_e_value;
      DAE.ComponentRef i_e_stateVar;
      DAE.Exp i_e_left;
      DAE.Exp i_e_right;
      DAE.ComponentRef i_left_componentRef;
      list<BackendDAE.WhenOperator> i_rest;
      Tpl.Text txt_8;
      Tpl.Text txt_7;
      Tpl.Text txt_6;
      Tpl.Text txt_5;
      Tpl.Text txt_4;
      Tpl.Text txt_3;
      Tpl.Text txt_2;
      Tpl.Text txt_1;
      Tpl.Text l_restbody;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           BackendDAE.ASSIGN(left = DAE.CREF(componentRef = i_left_componentRef), right = i_e_right) :: i_rest )
      equation
        l_restbody = dumpWhenOps(Tpl.emptyTxt, i_rest);
        txt = ExpressionDumpTpl.dumpCref(txt, i_left_componentRef);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(" = "));
        txt_1 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_right, "\"");
        txt = CodegenUtil.escapeCComments(txt, Tpl.textString(txt_1));
        txt = Tpl.writeTok(txt, Tpl.ST_LINE(";\n"));
        txt = Tpl.writeText(txt, l_restbody);
      then txt;

    case ( txt,
           BackendDAE.ASSIGN(left = i_e_left, right = i_e_right) :: i_rest )
      equation
        l_restbody = dumpWhenOps(Tpl.emptyTxt, i_rest);
        txt = ExpressionDumpTpl.dumpExp(txt, i_e_left, "\"");
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(" = "));
        txt_2 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_right, "\"");
        txt = CodegenUtil.escapeCComments(txt, Tpl.textString(txt_2));
        txt = Tpl.writeTok(txt, Tpl.ST_LINE(";\n"));
        txt = Tpl.writeText(txt, l_restbody);
      then txt;

    case ( txt,
           BackendDAE.REINIT(stateVar = i_e_stateVar, value = i_e_value) :: i_rest )
      equation
        l_restbody = dumpWhenOps(Tpl.emptyTxt, i_rest);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("reinit("));
        txt = ExpressionDumpTpl.dumpCref(txt, i_e_stateVar);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(",  "));
        txt_3 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_value, "\"");
        txt = CodegenUtil.escapeCComments(txt, Tpl.textString(txt_3));
        txt = Tpl.writeTok(txt, Tpl.ST_LINE(");\n"));
        txt = Tpl.writeText(txt, l_restbody);
      then txt;

    case ( txt,
           BackendDAE.ASSERT(condition = i_e_condition, message = i_e_message, level = i_e_level) :: i_rest )
      equation
        l_restbody = dumpWhenOps(Tpl.emptyTxt, i_rest);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("assert("));
        txt_4 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_condition, "\"");
        txt = CodegenUtil.escapeCComments(txt, Tpl.textString(txt_4));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(", "));
        txt_5 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_message, "\"");
        txt = CodegenUtil.escapeCComments(txt, Tpl.textString(txt_5));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(", "));
        txt_6 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_level, "\"");
        txt = CodegenUtil.escapeCComments(txt, Tpl.textString(txt_6));
        txt = Tpl.writeTok(txt, Tpl.ST_LINE(");\n"));
        txt = Tpl.writeText(txt, l_restbody);
      then txt;

    case ( txt,
           BackendDAE.TERMINATE(message = i_e_message) :: i_rest )
      equation
        l_restbody = dumpWhenOps(Tpl.emptyTxt, i_rest);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("terminate("));
        txt_7 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_message, "\"");
        txt = CodegenUtil.escapeCComments(txt, Tpl.textString(txt_7));
        txt = Tpl.writeTok(txt, Tpl.ST_LINE(")%>);\n"));
        txt = Tpl.writeText(txt, l_restbody);
      then txt;

    case ( txt,
           BackendDAE.NORETCALL(exp = i_e_exp) :: i_rest )
      equation
        l_restbody = dumpWhenOps(Tpl.emptyTxt, i_rest);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("noReturnCall("));
        txt_8 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_exp, "\"");
        txt = CodegenUtil.escapeCComments(txt, Tpl.textString(txt_8));
        txt = Tpl.writeTok(txt, Tpl.ST_LINE(")%>);\n"));
        txt = Tpl.writeText(txt, l_restbody);
      then txt;

    case ( txt,
           _ )
      equation
        txt = CodegenUtil.error(txt, Tpl.sourceInfo("CodegenUtilSimulation.tpl", 332, 14), "dumpEqs: Unknown equation");
      then txt;
  end match;
end dumpWhenOps;

protected function lm_68
  input Tpl.Text in_txt;
  input list<SimCodeVar.SimVar> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<SimCodeVar.SimVar> rest;
      DAE.ComponentRef i_cr;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           SimCodeVar.SIMVAR(name = i_cr) :: rest )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("<var>"));
        txt = ExpressionDumpTpl.dumpCref(txt, i_cr);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("</var>"));
        txt = Tpl.nextIter(txt);
        txt = lm_68(txt, rest);
      then txt;

    case ( txt,
           _ :: rest )
      equation
        txt = lm_68(txt, rest);
      then txt;
  end match;
end lm_68;

protected function lm_69
  input Tpl.Text in_txt;
  input list<DAE.Exp> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<DAE.Exp> rest;
      DAE.Exp i_exp;
      Tpl.Text txt_0;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_exp :: rest )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("<cell>"));
        txt_0 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_exp, "\"");
        txt = CodegenUtil.escapeCComments(txt, Tpl.textString(txt_0));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("</cell>"));
        txt = Tpl.nextIter(txt);
        txt = lm_69(txt, rest);
      then txt;
  end match;
end lm_69;

protected function fun_70
  input Tpl.Text in_txt;
  input SimCode.SimEqSystem in_a_eq;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_eq)
    local
      Tpl.Text txt;
      DAE.Exp i_e_exp;
      Tpl.Text txt_0;

    case ( txt,
           SimCode.SES_RESIDUAL(exp = i_e_exp) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("<residual>"));
        txt_0 = ExpressionDumpTpl.dumpExp(Tpl.emptyTxt, i_e_exp, "\"");
        txt = CodegenUtil.escapeCComments(txt, Tpl.textString(txt_0));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("</residual>"));
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end fun_70;

protected function lm_71
  input Tpl.Text in_txt;
  input list<tuple<Integer, Integer, SimCode.SimEqSystem>> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<tuple<Integer, Integer, SimCode.SimEqSystem>> rest;
      SimCode.SimEqSystem i_eq;
      Integer i_i2;
      Integer i_i1;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           (i_i1, i_i2, i_eq) :: rest )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("<cell row=\""));
        txt = Tpl.writeStr(txt, intString(i_i1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\" col=\""));
        txt = Tpl.writeStr(txt, intString(i_i2));
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("\">\n"));
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(2));
        txt = fun_70(txt, i_eq);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.popBlock(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("</cell>"));
        txt = lm_71(txt, rest);
      then txt;
  end match;
end lm_71;

protected function lm_72
  input Tpl.Text in_txt;
  input list<DAE.ComponentRef> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<DAE.ComponentRef> rest;
      DAE.ComponentRef i_cr;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_cr :: rest )
      equation
        txt = ExpressionDumpTpl.dumpCref(txt, i_cr);
        txt = Tpl.nextIter(txt);
        txt = lm_72(txt, rest);
      then txt;
  end match;
end lm_72;

protected function lm_73
  input Tpl.Text in_txt;
  input list<SimCode.SimEqSystem> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<SimCode.SimEqSystem> rest;
      SimCode.SimEqSystem i_eq;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_eq :: rest )
      equation
        txt = equationIndex(txt, i_eq);
        txt = Tpl.nextIter(txt);
        txt = lm_73(txt, rest);
      then txt;
  end match;
end lm_73;

protected function fun_74
  input Tpl.Text in_txt;
  input SimCode.SimEqSystem in_a_eq;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_eq)
    local
      Tpl.Text txt;
      list<SimCode.SimEqSystem> i_at_eqs;
      list<DAE.ComponentRef> i_at_crefs;
      Integer i_at_indexNonLinearSystem;
      list<tuple<Integer, Integer, SimCode.SimEqSystem>> i_at_simJac;
      list<DAE.Exp> i_at_beqs;
      list<SimCodeVar.SimVar> i_at_vars;
      SimCode.SimEqSystem i_e;

    case ( txt,
           (i_e as SimCode.SES_LINEAR(alternativeTearing = SOME(SimCode.LINEARSYSTEM(vars = i_at_vars, beqs = i_at_beqs, simJac = i_at_simJac)))) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("equation index: "));
        txt = equationIndexAlternativeTearing(txt, i_e);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "type: LINEAR\n",
                                    "\n"
                                }, true));
        txt = Tpl.pushIter(txt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_NEW_LINE()), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        txt = lm_68(txt, i_at_vars);
        txt = Tpl.popIter(txt);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("<row>\n"));
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(2));
        txt = Tpl.pushIter(txt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_NEW_LINE()), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        txt = lm_69(txt, i_at_beqs);
        txt = Tpl.popIter(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_NEW_LINE());
        txt = Tpl.popBlock(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "</row>\n",
                                    "<matrix>\n"
                                }, true));
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(2));
        txt = lm_71(txt, i_at_simJac);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.popBlock(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "</matrix>\n",
                                    "\n",
                                    "This is the alternative tearing set with casual solvability rules.\n",
                                    "If it fails, this function will call the strict tearing set."
                                }, false));
      then txt;

    case ( txt,
           (i_e as SimCode.SES_NONLINEAR(alternativeTearing = SOME(SimCode.NONLINEARSYSTEM(indexNonLinearSystem = i_at_indexNonLinearSystem, crefs = i_at_crefs, eqs = i_at_eqs)))) )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("equation index: "));
        txt = equationIndexAlternativeTearing(txt, i_e);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("indexNonlinear: "));
        txt = Tpl.writeStr(txt, intString(i_at_indexNonLinearSystem));
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "type: NONLINEAR\n",
                                    "\n",
                                    "vars: {"
                                }, false));
        txt = Tpl.pushIter(txt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_STRING(", ")), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        txt = lm_72(txt, i_at_crefs);
        txt = Tpl.popIter(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "}\n",
                                    "eqns: {"
                                }, false));
        txt = Tpl.pushIter(txt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_STRING(", ")), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        txt = lm_73(txt, i_at_eqs);
        txt = Tpl.popIter(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "}\n",
                                    "\n",
                                    "This is the alternative tearing set with casual solvability rules.\n",
                                    "If it fails, this function will call the strict tearing set."
                                }, false));
      then txt;

    case ( txt,
           _ )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("unknown equation"));
      then txt;
  end match;
end fun_74;

protected function lm_75
  input Tpl.Text in_txt;
  input list<SimCode.SimEqSystem> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<SimCode.SimEqSystem> rest;
      SimCode.SimEqSystem i_eq;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_eq :: rest )
      equation
        txt = fun_74(txt, i_eq);
        txt = lm_75(txt, rest);
      then txt;
  end match;
end lm_75;

public function dumpEqsAlternativeTearing
  input Tpl.Text txt;
  input list<SimCode.SimEqSystem> a_eqs;

  output Tpl.Text out_txt;
algorithm
  out_txt := lm_75(txt, a_eqs);
end dumpEqsAlternativeTearing;

annotation(__OpenModelica_Interface="backend");
end CodegenUtilSimulation;