encapsulated package Unparsing
"
  file:        Unparsing.mo
  package:     Unparsing
  description: Generated by Susan.
"

public import Tpl;

public import System;
public import Absyn;
public import AbsynUtil;
public import DAE;
public import SCode;
public import Util;
public import Types;

protected function lm_38
  input Tpl.Text in_txt;
  input SCode.Program in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      SCode.Program rest;
      SCode.Element i_cl;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           i_cl :: rest )
      equation
        txt = classExternalHeader(txt, i_cl, "");
        txt = lm_38(txt, rest);
      then txt;
  end match;
end lm_38;

public function programExternalHeader
  input Tpl.Text txt;
  input SCode.Program a_program;

  output Tpl.Text out_txt;
algorithm
  out_txt := Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                   "/* Automatically generated header for external MetaModelica functions */\n",
                                   "#ifdef __cplusplus\n",
                                   "extern \"C\" {\n",
                                   "#endif\n"
                               }, true));
  out_txt := lm_38(out_txt, a_program);
  out_txt := Tpl.softNewLine(out_txt);
  out_txt := Tpl.writeTok(out_txt, Tpl.ST_STRING_LIST({
                                       "#ifdef __cplusplus\n",
                                       "}\n",
                                       "#endif\n",
                                       "\n"
                                   }, true));
end programExternalHeader;

protected function lm_40
  input Tpl.Text in_txt;
  input list<DAE.Var> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<DAE.Var> rest;
      DAE.Ident i_var_name;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           DAE.TYPES_VAR(name = i_var_name) :: rest )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.writeStr(txt, i_var_name);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.nextIter(txt);
        txt = lm_40(txt, rest);
      then txt;

    case ( txt,
           _ :: rest )
      equation
        txt = lm_40(txt, rest);
      then txt;
  end match;
end lm_40;

protected function fun_41
  input Tpl.Text in_txt;
  input String in_mArg;
  input Tpl.Text in_a_fieldsStr;
  input Tpl.Text in_a_nElts;
  input Tpl.Text in_a_omcname;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_mArg, in_a_fieldsStr, in_a_nElts, in_a_omcname)
    local
      Tpl.Text txt;
      Tpl.Text a_fieldsStr;
      Tpl.Text a_nElts;
      Tpl.Text a_omcname;

    case ( txt,
           "0",
           _,
           _,
           a_omcname )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("ADD_METARECORD_DEFINITIONS const char* "));
        txt = Tpl.writeText(txt, a_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("__desc__fields[1] = {\"no fields\"};"));
      then txt;

    case ( txt,
           _,
           a_fieldsStr,
           a_nElts,
           a_omcname )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("ADD_METARECORD_DEFINITIONS const char* "));
        txt = Tpl.writeText(txt, a_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("__desc__fields["));
        txt = Tpl.writeText(txt, a_nElts);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("] = {"));
        txt = Tpl.writeText(txt, a_fieldsStr);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("};"));
      then txt;
  end match;
end fun_41;

protected function lm_42
  input Tpl.Text in_txt;
  input list<DAE.Type> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<DAE.Type> rest;
      Absyn.Path i_path;
      list<DAE.Var> i_ty_fields;
      String ret_9;
      String str_8;
      Tpl.Text l_fieldsDescription;
      Integer ret_6;
      Tpl.Text l_nElts;
      String ret_4;
      String ret_3;
      String ret_2;
      Tpl.Text l_omcname;
      Tpl.Text l_fieldsStr;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           DAE.T_METARECORD(fields = i_ty_fields, path = i_path) :: rest )
      equation
        l_fieldsStr = Tpl.pushIter(Tpl.emptyTxt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_STRING(",")), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        l_fieldsStr = lm_40(l_fieldsStr, i_ty_fields);
        l_fieldsStr = Tpl.popIter(l_fieldsStr);
        ret_2 = AbsynUtil.pathString(i_path, "$", false);
        ret_3 = System.stringReplace(ret_2, "_", "__");
        ret_4 = System.stringReplace(ret_3, "$", "_");
        l_omcname = Tpl.writeStr(Tpl.emptyTxt, ret_4);
        ret_6 = listLength(i_ty_fields);
        l_nElts = Tpl.writeStr(Tpl.emptyTxt, intString(ret_6));
        str_8 = Tpl.textString(l_nElts);
        l_fieldsDescription = fun_41(Tpl.emptyTxt, str_8, l_fieldsStr, l_nElts, l_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "#ifdef ADD_METARECORD_DEFINITIONS\n",
                                    "#ifndef "
                                }, false));
        txt = Tpl.writeText(txt, l_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "__desc_added\n",
                                    "#define "
                                }, false));
        txt = Tpl.writeText(txt, l_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("__desc_added\n"));
        txt = Tpl.writeText(txt, l_fieldsDescription);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("ADD_METARECORD_DEFINITIONS struct record_description "));
        txt = Tpl.writeText(txt, l_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("__desc = {\n"));
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(2));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.writeText(txt, l_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "\",\n",
                                    "\""
                                }, false));
        ret_9 = AbsynUtil.pathString(i_path, ".", false);
        txt = Tpl.writeStr(txt, ret_9);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("\",\n"));
        txt = Tpl.writeText(txt, l_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("__desc__fields\n"));
        txt = Tpl.popBlock(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "};\n",
                                    "#endif\n",
                                    "#else /* Only use the file as a header */\n",
                                    "extern struct record_description "
                                }, false));
        txt = Tpl.writeText(txt, l_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "__desc;\n",
                                    "#endif"
                                }, false));
        txt = Tpl.nextIter(txt);
        txt = lm_42(txt, rest);
      then txt;

    case ( txt,
           _ :: rest )
      equation
        txt = lm_42(txt, rest);
      then txt;
  end match;
end lm_42;

public function programExternalHeaderFromTypes
  input Tpl.Text txt;
  input list<DAE.Type> a_tys;

  output Tpl.Text out_txt;
algorithm
  out_txt := Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                   "/* Automatically generated header for bootstrapping MetaModelica */\n",
                                   "#ifdef __cplusplus\n",
                                   "extern \"C\" {\n",
                                   "#endif\n"
                               }, true));
  out_txt := Tpl.pushIter(out_txt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_NEW_LINE()), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
  out_txt := lm_42(out_txt, a_tys);
  out_txt := Tpl.popIter(out_txt);
  out_txt := Tpl.softNewLine(out_txt);
  out_txt := Tpl.writeTok(out_txt, Tpl.ST_STRING_LIST({
                                       "#ifdef __cplusplus\n",
                                       "}\n",
                                       "#endif\n",
                                       "\n"
                                   }, true));
end programExternalHeaderFromTypes;

protected function lm_44
  input Tpl.Text in_txt;
  input list<SCode.Element> in_items;
  input SCode.Ident in_a_c_name;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items, in_a_c_name)
    local
      Tpl.Text txt;
      list<SCode.Element> rest;
      SCode.Ident a_c_name;
      SCode.Element i_elt;

    case ( txt,
           {},
           _ )
      then txt;

    case ( txt,
           i_elt :: rest,
           a_c_name )
      equation
        txt = elementExternalHeader(txt, i_elt, a_c_name);
        txt = lm_44(txt, rest, a_c_name);
      then txt;
  end match;
end lm_44;

protected function fun_45
  input Tpl.Text in_txt;
  input SCode.Element in_a_cl;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_cl)
    local
      Tpl.Text txt;
      SCode.Ident i_c_name;
      list<SCode.Element> i_p_elementLst;

    case ( txt,
           SCode.CLASS(classDef = SCode.PARTS(elementLst = i_p_elementLst), name = i_c_name) )
      equation
        txt = lm_44(txt, i_p_elementLst, i_c_name);
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end fun_45;

public function classExternalHeader
  input Tpl.Text txt;
  input SCode.Element a_cl;
  input String a_pack;

  output Tpl.Text out_txt;
algorithm
  out_txt := fun_45(txt, a_cl);
end classExternalHeader;

public function pathString
  input Tpl.Text in_txt;
  input Absyn.Path in_a_path;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_path)
    local
      Tpl.Text txt;
      Absyn.Path i_path;
      Absyn.Ident i_name_1;
      String i_name;

    case ( txt,
           Absyn.IDENT(name = i_name) )
      equation
        txt = Tpl.writeStr(txt, i_name);
      then txt;

    case ( txt,
           Absyn.QUALIFIED(name = i_name_1, path = i_path) )
      equation
        txt = Tpl.writeStr(txt, i_name_1);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("."));
        txt = pathString(txt, i_path);
      then txt;

    case ( txt,
           Absyn.FULLYQUALIFIED(path = i_path) )
      equation
        txt = pathString(txt, i_path);
      then txt;

    case ( txt,
           _ )
      then txt;
  end match;
end pathString;

public function metaHelperBoxStart
  input Tpl.Text in_txt;
  input Integer in_a_numVariables;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_numVariables)
    local
      Tpl.Text txt;
      Integer i_numVariables;

    case ( txt,
           (i_numVariables as 0) )
      equation
        txt = Tpl.writeStr(txt, intString(i_numVariables));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
      then txt;

    case ( txt,
           (i_numVariables as 1) )
      equation
        txt = Tpl.writeStr(txt, intString(i_numVariables));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
      then txt;

    case ( txt,
           (i_numVariables as 2) )
      equation
        txt = Tpl.writeStr(txt, intString(i_numVariables));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
      then txt;

    case ( txt,
           (i_numVariables as 3) )
      equation
        txt = Tpl.writeStr(txt, intString(i_numVariables));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
      then txt;

    case ( txt,
           (i_numVariables as 4) )
      equation
        txt = Tpl.writeStr(txt, intString(i_numVariables));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
      then txt;

    case ( txt,
           (i_numVariables as 5) )
      equation
        txt = Tpl.writeStr(txt, intString(i_numVariables));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
      then txt;

    case ( txt,
           (i_numVariables as 6) )
      equation
        txt = Tpl.writeStr(txt, intString(i_numVariables));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
      then txt;

    case ( txt,
           (i_numVariables as 7) )
      equation
        txt = Tpl.writeStr(txt, intString(i_numVariables));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
      then txt;

    case ( txt,
           (i_numVariables as 8) )
      equation
        txt = Tpl.writeStr(txt, intString(i_numVariables));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
      then txt;

    case ( txt,
           (i_numVariables as 9) )
      equation
        txt = Tpl.writeStr(txt, intString(i_numVariables));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
      then txt;

    case ( txt,
           i_numVariables )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
        txt = Tpl.writeStr(txt, intString(i_numVariables));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(", "));
      then txt;
  end match;
end metaHelperBoxStart;

protected function lm_49
  input Tpl.Text in_txt;
  input list<SCode.Element> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<SCode.Element> rest;
      SCode.Ident i_name;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           SCode.COMPONENT(name = i_name) :: rest )
      equation
        txt = Tpl.writeStr(txt, i_name);
        txt = Tpl.nextIter(txt);
        txt = lm_49(txt, rest);
      then txt;

    case ( txt,
           _ :: rest )
      equation
        txt = lm_49(txt, rest);
      then txt;
  end match;
end lm_49;

protected function lm_50
  input Tpl.Text in_txt;
  input list<SCode.Element> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<SCode.Element> rest;
      SCode.Ident i_name;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           SCode.COMPONENT(name = i_name) :: rest )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.writeStr(txt, i_name);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.nextIter(txt);
        txt = lm_50(txt, rest);
      then txt;

    case ( txt,
           _ :: rest )
      equation
        txt = lm_50(txt, rest);
      then txt;
  end match;
end lm_50;

protected function fun_51
  input Tpl.Text in_txt;
  input String in_mArg;
  input Tpl.Text in_a_fieldsStr;
  input Tpl.Text in_a_nElts;
  input Tpl.Text in_a_omcname;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_mArg, in_a_fieldsStr, in_a_nElts, in_a_omcname)
    local
      Tpl.Text txt;
      Tpl.Text a_fieldsStr;
      Tpl.Text a_nElts;
      Tpl.Text a_omcname;

    case ( txt,
           "0",
           _,
           _,
           a_omcname )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("ADD_METARECORD_DEFINITIONS const char* "));
        txt = Tpl.writeText(txt, a_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("__desc__fields[1] = {\"no fields\"};"));
      then txt;

    case ( txt,
           _,
           a_fieldsStr,
           a_nElts,
           a_omcname )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("ADD_METARECORD_DEFINITIONS const char* "));
        txt = Tpl.writeText(txt, a_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("__desc__fields["));
        txt = Tpl.writeText(txt, a_nElts);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("] = {"));
        txt = Tpl.writeText(txt, a_fieldsStr);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("};"));
      then txt;
  end match;
end fun_51;

protected function fun_52
  input Tpl.Text in_txt;
  input list<SCode.Element> in_a_p_elementLst;
  input Tpl.Text in_a_fields;
  input Tpl.Text in_a_omcname;
  input Tpl.Text in_a_ctor;
  input Tpl.Text in_a_fullname;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_p_elementLst, in_a_fields, in_a_omcname, in_a_ctor, in_a_fullname)
    local
      Tpl.Text txt;
      Tpl.Text a_fields;
      Tpl.Text a_omcname;
      Tpl.Text a_ctor;
      Tpl.Text a_fullname;
      list<SCode.Element> i_p_elementLst;
      Integer ret_1;
      Integer ret_0;

    case ( txt,
           {},
           _,
           a_omcname,
           a_ctor,
           a_fullname )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("static const MMC_DEFSTRUCTLIT("));
        txt = Tpl.writeText(txt, a_fullname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("__struct,1,"));
        txt = Tpl.writeText(txt, a_ctor);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(") {&"));
        txt = Tpl.writeText(txt, a_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "__desc}};\n",
                                    "static void *"
                                }, false));
        txt = Tpl.writeText(txt, a_fullname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(" = MMC_REFSTRUCTLIT("));
        txt = Tpl.writeText(txt, a_fullname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("__struct);"));
        txt = Tpl.writeTok(txt, Tpl.ST_NEW_LINE());
      then txt;

    case ( txt,
           i_p_elementLst,
           a_fields,
           a_omcname,
           a_ctor,
           a_fullname )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("#define "));
        txt = Tpl.writeText(txt, a_fullname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
        txt = Tpl.writeText(txt, a_fields);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(") (mmc_mk_box"));
        ret_0 = listLength(i_p_elementLst);
        ret_1 = intAdd(1, ret_0);
        txt = metaHelperBoxStart(txt, ret_1);
        txt = Tpl.writeText(txt, a_ctor);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(",&"));
        txt = Tpl.writeText(txt, a_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("__desc,"));
        txt = Tpl.writeText(txt, a_fields);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("))"));
        txt = Tpl.writeTok(txt, Tpl.ST_NEW_LINE());
      then txt;
  end match;
end fun_52;

public function elementExternalHeader
  input Tpl.Text in_txt;
  input SCode.Element in_a_elt;
  input String in_a_pack;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_a_elt, in_a_pack)
    local
      Tpl.Text txt;
      String a_pack;
      SCode.Element i_elt;
      Integer i_r_index;
      SCode.Ident i_c_name;
      Absyn.Path i_r_name;
      list<SCode.Element> i_p_elementLst;
      String str_11;
      Tpl.Text l_fieldsDescription;
      Integer ret_9;
      Tpl.Text l_ctor;
      String ret_7;
      Tpl.Text l_fullname;
      Integer ret_5;
      Tpl.Text l_nElts;
      String ret_3;
      Tpl.Text l_omcname;
      Tpl.Text l_fieldsStr;
      Tpl.Text l_fields;

    case ( txt,
           SCode.CLASS(restriction = SCode.R_METARECORD(moved = true, name = i_r_name, index = i_r_index), classDef = SCode.PARTS(elementLst = i_p_elementLst), name = i_c_name),
           a_pack )
      equation
        l_fields = Tpl.pushIter(Tpl.emptyTxt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_STRING(",")), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        l_fields = lm_49(l_fields, i_p_elementLst);
        l_fields = Tpl.popIter(l_fields);
        l_fieldsStr = Tpl.pushIter(Tpl.emptyTxt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_STRING(",")), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        l_fieldsStr = lm_50(l_fieldsStr, i_p_elementLst);
        l_fieldsStr = Tpl.popIter(l_fieldsStr);
        l_omcname = Tpl.writeStr(Tpl.emptyTxt, a_pack);
        l_omcname = Tpl.writeTok(l_omcname, Tpl.ST_STRING("_"));
        l_omcname = pathString(l_omcname, i_r_name);
        l_omcname = Tpl.writeTok(l_omcname, Tpl.ST_STRING("_"));
        ret_3 = System.stringReplace(i_c_name, "_", "__");
        l_omcname = Tpl.writeStr(l_omcname, ret_3);
        ret_5 = listLength(i_p_elementLst);
        l_nElts = Tpl.writeStr(Tpl.emptyTxt, intString(ret_5));
        l_fullname = Tpl.writeStr(Tpl.emptyTxt, a_pack);
        l_fullname = Tpl.writeTok(l_fullname, Tpl.ST_STRING("__"));
        ret_7 = System.stringReplace(i_c_name, "_", "_5f");
        l_fullname = Tpl.writeStr(l_fullname, ret_7);
        ret_9 = intAdd(3, i_r_index);
        l_ctor = Tpl.writeStr(Tpl.emptyTxt, intString(ret_9));
        str_11 = Tpl.textString(l_nElts);
        l_fieldsDescription = fun_51(Tpl.emptyTxt, str_11, l_fieldsStr, l_nElts, l_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "#ifdef ADD_METARECORD_DEFINITIONS\n",
                                    "#ifndef "
                                }, false));
        txt = Tpl.writeText(txt, l_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "__desc_added\n",
                                    "#define "
                                }, false));
        txt = Tpl.writeText(txt, l_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("__desc_added\n"));
        txt = Tpl.writeText(txt, l_fieldsDescription);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("ADD_METARECORD_DEFINITIONS struct record_description "));
        txt = Tpl.writeText(txt, l_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("__desc = {\n"));
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(2));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("\""));
        txt = Tpl.writeText(txt, l_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "\",\n",
                                    "\""
                                }, false));
        txt = Tpl.writeStr(txt, a_pack);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("."));
        txt = pathString(txt, i_r_name);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("."));
        txt = Tpl.writeStr(txt, i_c_name);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("\",\n"));
        txt = Tpl.writeText(txt, l_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("__desc__fields\n"));
        txt = Tpl.popBlock(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "};\n",
                                    "#endif\n",
                                    "#else /* Only use the file as a header */\n",
                                    "extern struct record_description "
                                }, false));
        txt = Tpl.writeText(txt, l_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "__desc;\n",
                                    "#endif\n",
                                    "#define "
                                }, false));
        txt = Tpl.writeText(txt, l_fullname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("_3dBOX"));
        txt = Tpl.writeText(txt, l_nElts);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(" "));
        txt = Tpl.writeText(txt, l_ctor);
        txt = Tpl.softNewLine(txt);
        txt = fun_52(txt, i_p_elementLst, l_fields, l_omcname, l_ctor, l_fullname);
      then txt;

    case ( txt,
           (i_elt as SCode.CLASS(name = _)),
           a_pack )
      equation
        txt = classExternalHeader(txt, i_elt, a_pack);
      then txt;

    case ( txt,
           _,
           _ )
      then txt;
  end match;
end elementExternalHeader;

protected function lm_54
  input Tpl.Text in_txt;
  input SCode.Program in_items;
  input Tpl.Text in_a_buf2;
  input Tpl.Text in_a_buf1;

  output Tpl.Text out_txt;
  output Tpl.Text out_a_buf2;
  output Tpl.Text out_a_buf1;
algorithm
  (out_txt, out_a_buf2, out_a_buf1) :=
  match(in_txt, in_items, in_a_buf2, in_a_buf1)
    local
      Tpl.Text txt;
      SCode.Program rest;
      Tpl.Text a_buf2;
      Tpl.Text a_buf1;
      SCode.Element i_cl;

    case ( txt,
           {},
           a_buf2,
           a_buf1 )
      then (txt, a_buf2, a_buf1);

    case ( txt,
           i_cl :: rest,
           a_buf2,
           a_buf1 )
      equation
        (txt, a_buf1, a_buf2) = classExternalHeaderJulia(txt, a_buf1, a_buf2, i_cl, "");
        (txt, a_buf2, a_buf1) = lm_54(txt, rest, a_buf2, a_buf1);
      then (txt, a_buf2, a_buf1);
  end match;
end lm_54;

public function programExternalHeaderJulia
  input Tpl.Text txt;
  input SCode.Program a_program;

  output Tpl.Text out_txt;
protected
  Tpl.Text l_res;
  Tpl.Text l_buf2;
  Tpl.Text l_buf1;
algorithm
  l_buf1 := Tpl.emptyTxt;
  l_buf2 := Tpl.emptyTxt;
  (l_res, l_buf2, l_buf1) := lm_54(Tpl.emptyTxt, a_program, l_buf2, l_buf1);
  out_txt := Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                   "/* Automatically generated header for external MetaModelica functions */\n",
                                   "#include <julia.h>\n",
                                   "#include <assert.h>\n",
                                   "#ifdef __cplusplus\n",
                                   "extern \"C\" {\n",
                                   "#endif\n",
                                   "#ifdef ADD_METARECORD_DEFINITIONS\n"
                               }, true));
  out_txt := Tpl.writeText(out_txt, l_buf1);
  out_txt := Tpl.softNewLine(out_txt);
  out_txt := Tpl.writeTok(out_txt, Tpl.ST_STRING_LIST({
                                       "void OpenModelica_initAbsynReferences()\n",
                                       "{\n",
                                       "  /* Note: These values may be garbage collected away? Call this before each file is parsed? */\n"
                                   }, true));
  out_txt := Tpl.pushBlock(out_txt, Tpl.BT_INDENT(2));
  out_txt := Tpl.writeText(out_txt, l_buf2);
  out_txt := Tpl.softNewLine(out_txt);
  out_txt := Tpl.popBlock(out_txt);
  out_txt := Tpl.writeTok(out_txt, Tpl.ST_STRING_LIST({
                                       "}\n",
                                       "#else\n",
                                       "void OpenModelica_initAbsynReferences();\n",
                                       "#endif\n"
                                   }, true));
  out_txt := Tpl.writeText(out_txt, l_res);
  out_txt := Tpl.softNewLine(out_txt);
  out_txt := Tpl.writeTok(out_txt, Tpl.ST_STRING_LIST({
                                       "#ifdef __cplusplus\n",
                                       "}\n",
                                       "#endif\n",
                                       "\n"
                                   }, true));
end programExternalHeaderJulia;

protected function fun_56
  input Tpl.Text in_txt;
  input SCode.Element in_a_cl;
  input Tpl.Text in_a_buf1;
  input Tpl.Text in_a_buf2;
  input String in_a_pack;

  output Tpl.Text out_txt;
  output Tpl.Text out_a_buf1;
  output Tpl.Text out_a_buf2;
algorithm
  (out_txt, out_a_buf1, out_a_buf2) :=
  match(in_txt, in_a_cl, in_a_buf1, in_a_buf2, in_a_pack)
    local
      Tpl.Text txt;
      Tpl.Text a_buf1;
      Tpl.Text a_buf2;
      String a_pack;
      SCode.Element i_c;
      SCode.Ident i_c_name;

    case ( txt,
           (i_c as SCode.CLASS(name = i_c_name)),
           a_buf1,
           a_buf2,
           a_pack )
      equation
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING("jl_eval_string(\"using "));
        a_buf2 = Tpl.writeStr(a_buf2, i_c_name);
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING_LIST({
                                          "\");\n",
                                          "jl_module_t* "
                                      }, false));
        a_buf2 = Tpl.writeStr(a_buf2, i_c_name);
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING(" = (jl_module_t *) jl_eval_string(\""));
        a_buf2 = Tpl.writeStr(a_buf2, i_c_name);
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING_LIST({
                                          "\");\n",
                                          "if (!"
                                      }, false));
        a_buf2 = Tpl.writeStr(a_buf2, i_c_name);
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING_LIST({
                                          ")\n",
                                          "{\n"
                                      }, true));
        a_buf2 = Tpl.pushBlock(a_buf2, Tpl.BT_INDENT(2));
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING("fprintf(stderr, \"module "));
        a_buf2 = Tpl.writeStr(a_buf2, i_c_name);
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING_LIST({
                                          " not loaded, load it via using.\");\n",
                                          "fflush(NULL);\n"
                                      }, true));
        a_buf2 = Tpl.popBlock(a_buf2);
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING_LIST({
                                          "}\n",
                                          "assert(jl_is_module("
                                      }, false));
        a_buf2 = Tpl.writeStr(a_buf2, i_c_name);
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING("));"));
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_NEW_LINE());
        (txt, a_buf1, a_buf2) = classExternalHeaderJuliaWork(txt, a_buf1, a_buf2, i_c, a_pack);
      then (txt, a_buf1, a_buf2);

    case ( txt,
           _,
           a_buf1,
           a_buf2,
           _ )
      then (txt, a_buf1, a_buf2);
  end match;
end fun_56;

public function classExternalHeaderJulia
  input Tpl.Text txt;
  input Tpl.Text a_buf1;
  input Tpl.Text a_buf2;
  input SCode.Element a_cl;
  input String a_pack;

  output Tpl.Text out_txt;
  output Tpl.Text out_a_buf1;
  output Tpl.Text out_a_buf2;
algorithm
  (out_txt, out_a_buf1, out_a_buf2) := fun_56(txt, a_cl, a_buf1, a_buf2, a_pack);
end classExternalHeaderJulia;

protected function lm_58
  input Tpl.Text in_txt;
  input list<SCode.Element> in_items;
  input SCode.Ident in_a_c_name;
  input Tpl.Text in_a_buf2;
  input Tpl.Text in_a_buf1;

  output Tpl.Text out_txt;
  output Tpl.Text out_a_buf2;
  output Tpl.Text out_a_buf1;
algorithm
  (out_txt, out_a_buf2, out_a_buf1) :=
  match(in_txt, in_items, in_a_c_name, in_a_buf2, in_a_buf1)
    local
      Tpl.Text txt;
      list<SCode.Element> rest;
      SCode.Ident a_c_name;
      Tpl.Text a_buf2;
      Tpl.Text a_buf1;
      SCode.Element i_elt;

    case ( txt,
           {},
           _,
           a_buf2,
           a_buf1 )
      then (txt, a_buf2, a_buf1);

    case ( txt,
           i_elt :: rest,
           a_c_name,
           a_buf2,
           a_buf1 )
      equation
        (txt, a_buf1, a_buf2) = elementExternalHeaderJulia(txt, a_buf1, a_buf2, i_elt, a_c_name);
        (txt, a_buf2, a_buf1) = lm_58(txt, rest, a_c_name, a_buf2, a_buf1);
      then (txt, a_buf2, a_buf1);
  end match;
end lm_58;

protected function fun_59
  input Tpl.Text in_txt;
  input SCode.Element in_a_cl;
  input Tpl.Text in_a_buf1;
  input Tpl.Text in_a_buf2;

  output Tpl.Text out_txt;
  output Tpl.Text out_a_buf1;
  output Tpl.Text out_a_buf2;
algorithm
  (out_txt, out_a_buf1, out_a_buf2) :=
  match(in_txt, in_a_cl, in_a_buf1, in_a_buf2)
    local
      Tpl.Text txt;
      Tpl.Text a_buf1;
      Tpl.Text a_buf2;
      SCode.Ident i_c_name;
      list<SCode.Element> i_p_elementLst;

    case ( txt,
           SCode.CLASS(classDef = SCode.PARTS(elementLst = i_p_elementLst), name = i_c_name),
           a_buf1,
           a_buf2 )
      equation
        (txt, a_buf2, a_buf1) = lm_58(txt, i_p_elementLst, i_c_name, a_buf2, a_buf1);
      then (txt, a_buf1, a_buf2);

    case ( txt,
           _,
           a_buf1,
           a_buf2 )
      then (txt, a_buf1, a_buf2);
  end match;
end fun_59;

public function classExternalHeaderJuliaWork
  input Tpl.Text txt;
  input Tpl.Text a_buf1;
  input Tpl.Text a_buf2;
  input SCode.Element a_cl;
  input String a_pack;

  output Tpl.Text out_txt;
  output Tpl.Text out_a_buf1;
  output Tpl.Text out_a_buf2;
algorithm
  (out_txt, out_a_buf1, out_a_buf2) := fun_59(txt, a_cl, a_buf1, a_buf2);
end classExternalHeaderJuliaWork;

protected function lm_61
  input Tpl.Text in_txt;
  input list<SCode.Element> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<SCode.Element> rest;
      SCode.Ident i_name;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           SCode.COMPONENT(name = i_name) :: rest )
      equation
        txt = Tpl.writeStr(txt, i_name);
        txt = Tpl.nextIter(txt);
        txt = lm_61(txt, rest);
      then txt;

    case ( txt,
           _ :: rest )
      equation
        txt = lm_61(txt, rest);
      then txt;
  end match;
end lm_61;

protected function lm_62
  input Tpl.Text in_txt;
  input list<SCode.Element> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<SCode.Element> rest;
      SCode.Ident i_name;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           SCode.COMPONENT(name = i_name) :: rest )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(", "));
        txt = Tpl.writeStr(txt, i_name);
        txt = lm_62(txt, rest);
      then txt;

    case ( txt,
           _ :: rest )
      equation
        txt = lm_62(txt, rest);
      then txt;
  end match;
end lm_62;

protected function lm_63
  input Tpl.Text in_txt;
  input list<SCode.Element> in_items;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_items)
    local
      Tpl.Text txt;
      list<SCode.Element> rest;
      SCode.Ident i_name;

    case ( txt,
           {} )
      then txt;

    case ( txt,
           SCode.COMPONENT(name = i_name) :: rest )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("jl_value_t *"));
        txt = Tpl.writeStr(txt, i_name);
        txt = Tpl.nextIter(txt);
        txt = lm_63(txt, rest);
      then txt;

    case ( txt,
           _ :: rest )
      equation
        txt = lm_63(txt, rest);
      then txt;
  end match;
end lm_63;

protected function fun_64
  input Tpl.Text in_txt;
  input Integer in_mArg;
  input Tpl.Text in_a_fields1;
  input Tpl.Text in_a_fields2;
  input list<SCode.Element> in_a_p_elementLst;
  input Tpl.Text in_a_fieldsWithType;
  input Tpl.Text in_a_omcname;
  input Tpl.Text in_a_fullname;

  output Tpl.Text out_txt;
algorithm
  out_txt :=
  match(in_txt, in_mArg, in_a_fields1, in_a_fields2, in_a_p_elementLst, in_a_fieldsWithType, in_a_omcname, in_a_fullname)
    local
      Tpl.Text txt;
      Tpl.Text a_fields1;
      Tpl.Text a_fields2;
      list<SCode.Element> a_p_elementLst;
      Tpl.Text a_fieldsWithType;
      Tpl.Text a_omcname;
      Tpl.Text a_fullname;
      Integer ret_4;
      Integer ret_3;
      Integer ret_2;
      Integer ret_1;
      Integer ret_0;

    case ( txt,
           0,
           _,
           _,
           _,
           _,
           a_omcname,
           a_fullname )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("#define "));
        txt = Tpl.writeText(txt, a_fullname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(" jl_call0("));
        txt = Tpl.writeText(txt, a_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(")"));
      then txt;

    case ( txt,
           1,
           _,
           a_fields2,
           a_p_elementLst,
           a_fieldsWithType,
           a_omcname,
           a_fullname )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("static inline jl_value_t* "));
        txt = Tpl.writeText(txt, a_fullname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
        txt = Tpl.writeText(txt, a_fieldsWithType);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE(") {\n"));
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(2));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("return jl_call"));
        ret_0 = listLength(a_p_elementLst);
        txt = Tpl.writeStr(txt, intString(ret_0));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
        txt = Tpl.writeText(txt, a_omcname);
        txt = Tpl.writeText(txt, a_fields2);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE(");\n"));
        txt = Tpl.popBlock(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("}"));
      then txt;

    case ( txt,
           2,
           _,
           a_fields2,
           a_p_elementLst,
           a_fieldsWithType,
           a_omcname,
           a_fullname )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("static inline jl_value_t* "));
        txt = Tpl.writeText(txt, a_fullname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
        txt = Tpl.writeText(txt, a_fieldsWithType);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE(") {\n"));
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(2));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("return jl_call"));
        ret_1 = listLength(a_p_elementLst);
        txt = Tpl.writeStr(txt, intString(ret_1));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
        txt = Tpl.writeText(txt, a_omcname);
        txt = Tpl.writeText(txt, a_fields2);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE(");\n"));
        txt = Tpl.popBlock(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("}"));
      then txt;

    case ( txt,
           3,
           _,
           a_fields2,
           a_p_elementLst,
           a_fieldsWithType,
           a_omcname,
           a_fullname )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("static inline jl_value_t* "));
        txt = Tpl.writeText(txt, a_fullname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
        txt = Tpl.writeText(txt, a_fieldsWithType);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE(") {\n"));
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(2));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("return jl_call"));
        ret_2 = listLength(a_p_elementLst);
        txt = Tpl.writeStr(txt, intString(ret_2));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
        txt = Tpl.writeText(txt, a_omcname);
        txt = Tpl.writeText(txt, a_fields2);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE(");\n"));
        txt = Tpl.popBlock(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("}"));
      then txt;

    case ( txt,
           _,
           a_fields1,
           _,
           a_p_elementLst,
           a_fieldsWithType,
           a_omcname,
           a_fullname )
      equation
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("static inline jl_value_t* "));
        txt = Tpl.writeText(txt, a_fullname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("("));
        txt = Tpl.writeText(txt, a_fieldsWithType);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE(") {\n"));
        txt = Tpl.pushBlock(txt, Tpl.BT_INDENT(2));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("jl_value_t *values["));
        ret_3 = listLength(a_p_elementLst);
        txt = Tpl.writeStr(txt, intString(ret_3));
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("] = {"));
        txt = Tpl.writeText(txt, a_fields1);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    "};\n",
                                    "return jl_call("
                                }, false));
        txt = Tpl.writeText(txt, a_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(", values, "));
        ret_4 = listLength(a_p_elementLst);
        txt = Tpl.writeStr(txt, intString(ret_4));
        txt = Tpl.writeTok(txt, Tpl.ST_LINE(");\n"));
        txt = Tpl.popBlock(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("}"));
      then txt;
  end match;
end fun_64;

protected function fun_65
  input Tpl.Text in_txt;
  input SCode.Element in_a_elt;
  input Tpl.Text in_a_buf1;
  input Tpl.Text in_a_buf2;
  input String in_a_pack;

  output Tpl.Text out_txt;
  output Tpl.Text out_a_buf1;
  output Tpl.Text out_a_buf2;
algorithm
  (out_txt, out_a_buf1, out_a_buf2) :=
  match(in_txt, in_a_elt, in_a_buf1, in_a_buf2, in_a_pack)
    local
      Tpl.Text txt;
      Tpl.Text a_buf1;
      Tpl.Text a_buf2;
      String a_pack;
      SCode.Element i_elt;
      Absyn.Path i_r_name;
      list<SCode.Element> i_p_elementLst;
      SCode.Ident i_c_name;
      Integer ret_8;
      String ret_7;
      Tpl.Text l_fullname;
      String ret_5;
      Tpl.Text l_funcName;
      Tpl.Text l_fieldsWithType;
      Tpl.Text l_fields2;
      Tpl.Text l_fields1;
      Tpl.Text l_omcname;

    case ( txt,
           SCode.CLASS(restriction = SCode.R_UNIONTYPE(), classDef = SCode.PARTS(elementLst = _), name = i_c_name),
           a_buf1,
           a_buf2,
           a_pack )
      equation
        l_omcname = Tpl.writeStr(Tpl.emptyTxt, a_pack);
        l_omcname = Tpl.writeTok(l_omcname, Tpl.ST_STRING("_"));
        l_omcname = Tpl.writeStr(l_omcname, i_c_name);
        a_buf1 = Tpl.writeTok(a_buf1, Tpl.ST_STRING("jl_value_t *"));
        a_buf1 = Tpl.writeText(a_buf1, l_omcname);
        a_buf1 = Tpl.writeTok(a_buf1, Tpl.ST_LINE(" = NULL;\n"));
        a_buf1 = Tpl.writeTok(a_buf1, Tpl.ST_NEW_LINE());
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING("assert(("));
        a_buf2 = Tpl.writeText(a_buf2, l_omcname);
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING(" = jl_get_global("));
        a_buf2 = Tpl.writeStr(a_buf2, a_pack);
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING(", jl_symbol(\""));
        a_buf2 = Tpl.writeStr(a_buf2, i_c_name);
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING("\"))));"));
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_NEW_LINE());
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("extern jl_value_t *"));
        txt = Tpl.writeText(txt, l_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING(";"));
        txt = Tpl.writeTok(txt, Tpl.ST_NEW_LINE());
      then (txt, a_buf1, a_buf2);

    case ( txt,
           SCode.CLASS(restriction = SCode.R_METARECORD(moved = true, name = i_r_name), classDef = SCode.PARTS(elementLst = i_p_elementLst), name = i_c_name),
           a_buf1,
           a_buf2,
           a_pack )
      equation
        l_fields1 = Tpl.pushIter(Tpl.emptyTxt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_STRING(",")), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        l_fields1 = lm_61(l_fields1, i_p_elementLst);
        l_fields1 = Tpl.popIter(l_fields1);
        l_fields2 = lm_62(Tpl.emptyTxt, i_p_elementLst);
        l_fieldsWithType = Tpl.pushIter(Tpl.emptyTxt, Tpl.ITER_OPTIONS(0, NONE(), SOME(Tpl.ST_STRING(",")), 0, 0, Tpl.ST_NEW_LINE(), 0, Tpl.ST_NEW_LINE()));
        l_fieldsWithType = lm_63(l_fieldsWithType, i_p_elementLst);
        l_fieldsWithType = Tpl.popIter(l_fieldsWithType);
        l_funcName = Tpl.writeStr(Tpl.emptyTxt, a_pack);
        l_funcName = Tpl.writeTok(l_funcName, Tpl.ST_STRING("."));
        l_funcName = pathString(l_funcName, i_r_name);
        l_omcname = Tpl.writeStr(Tpl.emptyTxt, a_pack);
        l_omcname = Tpl.writeTok(l_omcname, Tpl.ST_STRING("_"));
        l_omcname = pathString(l_omcname, i_r_name);
        l_omcname = Tpl.writeTok(l_omcname, Tpl.ST_STRING("_"));
        ret_5 = System.stringReplace(i_c_name, "_", "__");
        l_omcname = Tpl.writeStr(l_omcname, ret_5);
        l_fullname = Tpl.writeStr(Tpl.emptyTxt, a_pack);
        l_fullname = Tpl.writeTok(l_fullname, Tpl.ST_STRING("__"));
        ret_7 = System.stringReplace(i_c_name, "_", "_5f");
        l_fullname = Tpl.writeStr(l_fullname, ret_7);
        a_buf1 = Tpl.writeTok(a_buf1, Tpl.ST_STRING("jl_function_t *"));
        a_buf1 = Tpl.writeText(a_buf1, l_omcname);
        a_buf1 = Tpl.writeTok(a_buf1, Tpl.ST_STRING_LIST({
                                          " = NULL;\n",
                                          "jl_value_t *"
                                      }, false));
        a_buf1 = Tpl.writeText(a_buf1, l_omcname);
        a_buf1 = Tpl.writeTok(a_buf1, Tpl.ST_LINE("_type = NULL;\n"));
        a_buf1 = Tpl.writeTok(a_buf1, Tpl.ST_NEW_LINE());
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING("assert(("));
        a_buf2 = Tpl.writeText(a_buf2, l_omcname);
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING(" = jl_get_function("));
        a_buf2 = Tpl.writeStr(a_buf2, a_pack);
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING(", \""));
        a_buf2 = Tpl.writeStr(a_buf2, i_c_name);
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING_LIST({
                                          "\")));\n",
                                          "assert(("
                                      }, false));
        a_buf2 = Tpl.writeText(a_buf2, l_omcname);
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING("_type = jl_get_global("));
        a_buf2 = Tpl.writeStr(a_buf2, a_pack);
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING(", jl_symbol(\""));
        a_buf2 = Tpl.writeStr(a_buf2, i_c_name);
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_STRING("\"))));"));
        a_buf2 = Tpl.writeTok(a_buf2, Tpl.ST_NEW_LINE());
        txt = Tpl.writeTok(txt, Tpl.ST_STRING("extern jl_function_t *"));
        txt = Tpl.writeText(txt, l_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_STRING_LIST({
                                    ";\n",
                                    "extern jl_function_t *"
                                }, false));
        txt = Tpl.writeText(txt, l_omcname);
        txt = Tpl.writeTok(txt, Tpl.ST_LINE("_type;\n"));
        ret_8 = listLength(i_p_elementLst);
        txt = fun_64(txt, ret_8, l_fields1, l_fields2, i_p_elementLst, l_fieldsWithType, l_omcname, l_fullname);
        txt = Tpl.softNewLine(txt);
        txt = Tpl.writeTok(txt, Tpl.ST_NEW_LINE());
      then (txt, a_buf1, a_buf2);

    case ( txt,
           (i_elt as SCode.CLASS(name = _)),
           a_buf1,
           a_buf2,
           a_pack )
      equation
        (txt, a_buf1, a_buf2) = classExternalHeaderJuliaWork(txt, a_buf1, a_buf2, i_elt, a_pack);
      then (txt, a_buf1, a_buf2);

    case ( txt,
           _,
           a_buf1,
           a_buf2,
           _ )
      then (txt, a_buf1, a_buf2);
  end match;
end fun_65;

public function elementExternalHeaderJulia
  input Tpl.Text txt;
  input Tpl.Text a_buf1;
  input Tpl.Text a_buf2;
  input SCode.Element a_elt;
  input String a_pack;

  output Tpl.Text out_txt;
  output Tpl.Text out_a_buf1;
  output Tpl.Text out_a_buf2;
algorithm
  (out_txt, out_a_buf1, out_a_buf2) := fun_65(txt, a_elt, a_buf1, a_buf2, a_pack);
end elementExternalHeaderJulia;

annotation(__OpenModelica_Interface="backend");
end Unparsing;